@@ -1,6 +1,5 @@
 import pytest
 from ctypes import *
-import struct
 
 @pytest.mark.pypy_only
 def test_get_ffi_argtype():
@@ -15,11 +14,11 @@ def test_get_ffi_argtype():
     (c_short,  int),
     (c_ushort, int),
     (c_int, int),
-    (c_uint, int),
+    (c_uint, long),
     (c_long, int),
-    (c_ulong, int),
-    (c_longlong, int),
-    (c_ulonglong, int),
+    (c_ulong, long),
+    (c_longlong, long),
+    (c_ulonglong, long),
     (c_double, float),
     (c_float, float),
 ])
@@ -80,21 +79,3 @@ def test_issue2813_cant_change_fields_after_get_ffi_argtype():
     ffitype = C.get_ffi_argtype()
     with pytest.raises(NotImplementedError):
         C._fields_ = [('x', c_int)]
-
-def test_memoryview():
-    x = c_int(32)
-    p1 = pointer(x)
-    p2 = pointer(p1)
-
-    m1 = memoryview(p1)
-    assert struct.unpack('P', m1)[0] == addressof(x)
-    m2 = memoryview(p2)
-    assert struct.unpack('P', m2)[0] == addressof(p1)
-
-def test_pointer_from_array():
-    A = c_ubyte * 4
-    a = A(19, 72, 0, 23)
-    P = POINTER(c_ubyte)
-    p = P(a)
-    for i in range(len(a)):
-        assert p[i] == a[i]
