@@ -24,6 +24,7 @@
 
 import unittest
 import sqlite3 as sqlite
+from test import test_support
 
 def func_returntext():
     return "foo"
@@ -36,7 +37,8 @@ def func_returnfloat():
 def func_returnnull():
     return None
 def func_returnblob():
-    return buffer("blob")
+    with test_support.check_py3k_warnings():
+        return buffer("blob")
 def func_returnlonglong():
     return 1<<31
 def func_raiseexception():
@@ -202,8 +204,9 @@ class FunctionTests(unittest.TestCase):
         cur = self.con.cursor()
         cur.execute("select returnblob()")
         val = cur.fetchone()[0]
-        self.assertEqual(type(val), buffer)
-        self.assertEqual(val, buffer("blob"))
+        with test_support.check_py3k_warnings():
+            self.assertEqual(type(val), buffer)
+            self.assertEqual(val, buffer("blob"))
 
     def CheckFuncReturnLongLong(self):
         cur = self.con.cursor()
@@ -246,7 +249,8 @@ class FunctionTests(unittest.TestCase):
 
     def CheckParamBlob(self):
         cur = self.con.cursor()
-        cur.execute("select isblob(?)", (buffer("blob"),))
+        with test_support.check_py3k_warnings():
+            cur.execute("select isblob(?)", (buffer("blob"),))
         val = cur.fetchone()[0]
         self.assertEqual(val, 1)
 
@@ -269,8 +273,9 @@ class AggregateTests(unittest.TestCase):
                 b blob
                 )
             """)
-        cur.execute("insert into test(t, i, f, n, b) values (?, ?, ?, ?, ?)",
-            ("foo", 5, 3.14, None, buffer("blob"),))
+        with test_support.check_py3k_warnings():
+            cur.execute("insert into test(t, i, f, n, b) values (?, ?, ?, ?, ?)",
+                ("foo", 5, 3.14, None, buffer("blob"),))
 
         self.con.create_aggregate("nostep", 1, AggrNoStep)
         self.con.create_aggregate("nofinalize", 1, AggrNoFinalize)
@@ -364,7 +369,8 @@ class AggregateTests(unittest.TestCase):
 
     def CheckAggrCheckParamBlob(self):
         cur = self.con.cursor()
-        cur.execute("select checkType('blob', ?)", (buffer("blob"),))
+        with test_support.check_py3k_warnings():
+            cur.execute("select checkType('blob', ?)", (buffer("blob"),))
         val = cur.fetchone()[0]
         self.assertEqual(val, 1)
 
