@@ -665,6 +665,20 @@ class MmapTests(unittest.TestCase):
         finally:
             s.close()
 
+    @unittest.skipIf(os.name == 'nt', 'cannot resize anonymous mmaps on Windows')
+    def test_resize_past_pos(self):
+        m = mmap.mmap(-1, 8192)
+        self.addCleanup(m.close)
+        m.read(5000)
+        try:
+            m.resize(4096)
+        except SystemError:
+            self.skipTest("resizing not supported")
+        self.assertEqual(m.read(14), '')
+        self.assertRaises(ValueError, m.read_byte)
+        self.assertRaises(ValueError, m.write_byte, 'b')
+        self.assertRaises(ValueError, m.write, 'abc')
+
 
 class LargeMmapTests(unittest.TestCase):
 
