@@ -7,6 +7,7 @@ Generate Python bytecode from a Abstract Syntax Tree.
 # you figure out a way to remove them, great, but try a translation first,
 # please.
 
+from rpython.rlib.objectmodel import specialize
 from pypy.interpreter.astcompiler import ast, assemble, symtable, consts, misc
 from pypy.interpreter.astcompiler import optimize # For side effects
 from pypy.interpreter.pyparser.error import SyntaxError
@@ -321,7 +322,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
     def visit_ClassDef(self, cls):
         self.update_position(cls.lineno, True)
         self.visit_sequence(cls.decorator_list)
-        self.load_const(self.space.wrap(cls.name))
+        self.load_const(self.space.newtext(cls.name))
         self.visit_sequence(cls.bases)
         bases_count = len(cls.bases) if cls.bases is not None else 0
         self.emit_op_arg(ops.BUILD_TUPLE, bases_count)
@@ -564,7 +565,6 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
             self.emit_jump(ops.JUMP_FORWARD, end)
             self.use_next_block(next_except)
         self.emit_op(ops.END_FINALLY)   # this END_FINALLY will always re-raise
-        self.is_dead_code()
         self.use_next_block(otherwise)
         self.visit_sequence(te.orelse)
         self.use_next_block(end)
@@ -611,7 +611,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
                 level = 0
             else:
                 level = -1
-            self.load_const(self.space.wrap(level))
+            self.load_const(self.space.newint(level))
             self.load_const(self.space.w_None)
             self.emit_op_name(ops.IMPORT_NAME, self.names, alias.name)
             # If there's no asname then we store the root module.  If there is
@@ -654,12 +654,12 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
             level = -1
         else:
             level = imp.level
-        self.load_const(space.wrap(level))
+        self.load_const(space.newint(level))
         names_w = [None]*len(imp.names)
         for i in range(len(imp.names)):
             alias = imp.names[i]
             assert isinstance(alias, ast.alias)
-            names_w[i] = space.wrap(alias.name)
+            names_w[i] = space.newtext(alias.name)
         self.load_const(space.newtuple(names_w))
         if imp.module:
             mod_name = imp.module
@@ -944,7 +944,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         self.name_op(name.id, name.ctx)
 
     def visit_keyword(self, keyword):
-        self.load_const(self.space.wrap(keyword.arg))
+        self.load_const(self.space.newtext(keyword.arg))
         keyword.value.walkabout(self)
 
     def visit_Call(self, call):
@@ -1028,11 +1028,17 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
 
     def visit_ListComp(self, lc):
         self.update_position(lc.lineno)
-        if len(lc.generators) != 1 or lc.generators[0].ifs:
+        if len(lc.generators) == 1:
+            comp = lc.generators[0]
+            assert isinstance(comp, ast.comprehension)
+            if comp.ifs:
+                single = False
+                self.emit_op_arg(ops.BUILD_LIST, 0)
+            else:
+                single = True
+        else:
             single = False
             self.emit_op_arg(ops.BUILD_LIST, 0)
-        else:
-            single = True
         self._listcomp_generator(lc.generators, 0, lc.elt, single=single)
 
     def _comp_generator(self, node, generators, gen_index):
@@ -1197,6 +1203,20 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
             sub.value.walkabout(self)
         self._compile_slice(sub.slice, sub.ctx)
 
+    def _revdb_metavar(self, node):
+        # moved in its own function for the import statement
+        from pypy.interpreter.reverse_debugging import dbstate
+        if not dbstate.standard_code:
+            self.emit_op_arg(ops.LOAD_REVDB_VAR, node.metavar)
+            return True
+        return False
+
+    def visit_RevDBMetaVar(self, node):
+        if self.space.reverse_debugging and self._revdb_metavar(node):
+            return
+        self.error("Unknown character ('$NUM' is only valid in the "
+                   "reverse-debugger)", node)
+
 
 class TopLevelCodeGenerator(PythonCodeGenerator):
 
