@@ -39,7 +39,7 @@ class W_BytesBuilder(W_Root):
             raise oefmt(space.w_ValueError, "no length of built builder")
         return space.newint(self.builder.getlength())
 
-W_BytesBuilder.typedef = TypeDef("BytesBuilder",
+W_BytesBuilder.typedef = TypeDef("StringBuilder",
     __new__ = interp2app(func_with_new_name(
                                 W_BytesBuilder.descr__new__.im_func,
                                 'BytesBuilder_new')),
@@ -49,6 +49,7 @@ W_BytesBuilder.typedef = TypeDef("BytesBuilder",
     __len__ = interp2app(W_BytesBuilder.descr_len),
 )
 W_BytesBuilder.typedef.acceptable_as_base_class = False
+W_StringBuilder = W_BytesBuilder
 
 class W_UnicodeBuilder(W_Root):
     def __init__(self, space, size):
@@ -83,7 +84,7 @@ class W_UnicodeBuilder(W_Root):
             raise oefmt(space.w_ValueError, "no length of built builder")
         return space.newint(self.builder.getlength())
 
-W_UnicodeBuilder.typedef = TypeDef("StringBuilder",
+W_UnicodeBuilder.typedef = TypeDef("UnicodeBuilder",
     __new__ = interp2app(func_with_new_name(
                                 W_UnicodeBuilder.descr__new__.im_func,
                                 'UnicodeBuilder_new')),
@@ -93,4 +94,3 @@ W_UnicodeBuilder.typedef = TypeDef("StringBuilder",
     __len__ = interp2app(W_UnicodeBuilder.descr_len),
 )
 W_UnicodeBuilder.typedef.acceptable_as_base_class = False
-W_StringBuilder = W_UnicodeBuilder
