@@ -13,13 +13,13 @@ from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import oefmt, wrap_oserror
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import GetSetProperty, TypeDef
+from pypy.module._multiprocessing.interp_connection import w_handle
 
 RECURSIVE_MUTEX, SEMAPHORE = range(2)
-sys_platform = sys.platform
 
 if sys.platform == 'win32':
     from rpython.rlib import rwin32
-    from pypy.module._multiprocessing.interp_win32_py3 import (
+    from pypy.module._multiprocessing.interp_win32 import (
         _GetTickCount, handle_w)
 
     SEM_VALUE_MAX = sys.maxint
@@ -33,10 +33,8 @@ if sys.platform == 'win32':
     _ReleaseSemaphore = rwin32.winexternal(
         'ReleaseSemaphore', [rwin32.HANDLE, rffi.LONG, rffi.LONGP],
         rwin32.BOOL,
-        save_err=rffi.RFFI_SAVE_LASTERROR, releasegil=False)
+        save_err=rffi.RFFI_SAVE_LASTERROR)
 
-    def sem_unlink(name):
-        return None
 else:
     from rpython.rlib import rposix
 
@@ -48,8 +46,7 @@ else:
     eci = ExternalCompilationInfo(
         includes = ['sys/time.h',
                     'limits.h',
-                    'semaphore.h',
-                    ],
+                    'semaphore.h'],
         libraries = libraries,
         )
 
@@ -100,7 +97,7 @@ else:
                          save_err=rffi.RFFI_SAVE_ERRNO)
     _sem_trywait = external('sem_trywait', [SEM_T], rffi.INT,
                             save_err=rffi.RFFI_SAVE_ERRNO)
-    _sem_post = external('sem_post', [SEM_T], rffi.INT, releasegil=False,
+    _sem_post = external('sem_post', [SEM_T], rffi.INT,
                          save_err=rffi.RFFI_SAVE_ERRNO)
     _sem_getvalue = external('sem_getvalue', [SEM_T, rffi.INTP], rffi.INT,
                              save_err=rffi.RFFI_SAVE_ERRNO)
@@ -216,14 +213,6 @@ else:
     def handle_w(space, w_handle):
         return rffi.cast(SEM_T, space.int_w(w_handle))
 
-# utilized by POSIX and win32
-def semaphore_unlink(space, w_name):
-    name = space.text_w(w_name)
-    try:
-        sem_unlink(name)
-    except OSError as e:
-        raise wrap_oserror(space, e)
-
 class CounterState:
     def __init__(self, space):
         self.counter = 0
@@ -270,8 +259,6 @@ if sys.platform == 'win32':
         res = rwin32.WaitForSingleObject(self.handle, 0)
 
         if res != rwin32.WAIT_TIMEOUT:
-            self.last_tid = rthread.get_ident()
-            self.count += 1
             return True
 
         msecs = full_msecs
@@ -304,8 +291,6 @@ if sys.platform == 'win32':
 
         # handle result
         if res != rwin32.WAIT_TIMEOUT:
-            self.last_tid = rthread.get_ident()
-            self.count += 1
             return True
         return False
 
@@ -336,12 +321,12 @@ if sys.platform == 'win32':
 else:
     def create_semaphore(space, name, val, max):
         sem = sem_open(name, os.O_CREAT | os.O_EXCL, 0600, val)
-        rgc.add_memory_pressure(SEM_T_SIZE)
-        return sem
-
-    def reopen_semaphore(name):
-        sem = sem_open(name, 0, 0600, 0)
-        rgc.add_memory_pressure(SEM_T_SIZE)
+        try:
+            sem_unlink(name)
+        except OSError:
+            pass
+        else:
+            rgc.add_memory_pressure(SEM_T_SIZE)
         return sem
 
     def delete_semaphore(handle):
@@ -385,8 +370,7 @@ else:
                         return False
                     raise
                 _check_signals(space)
-                self.last_tid = rthread.get_ident()
-                self.count += 1
+
                 return True
         finally:
             if deadline:
@@ -449,29 +433,19 @@ else:
 
 
 class W_SemLock(W_Root):
-    def __init__(self, space, handle, kind, maxvalue, name):
+    def __init__(self, space, handle, kind, maxvalue):
         self.handle = handle
         self.kind = kind
         self.count = 0
         self.maxvalue = maxvalue
         self.register_finalizer(space)
-        self.last_tid = -1
-        self.name = name
-
-    def name_get(self, space):
-        if self.name is None:
-            return space.w_None
-        return space.newtext(self.name)
 
     def kind_get(self, space):
         return space.newint(self.kind)
-
     def maxvalue_get(self, space):
         return space.newint(self.maxvalue)
-
     def handle_get(self, space):
-        h = rffi.cast(rffi.INTPTR_T, self.handle)
-        return space.newint(h)
+        return w_handle(space, self.handle)
 
     def get_count(self, space):
         return space.newint(self.count)
@@ -502,15 +476,15 @@ class W_SemLock(W_Root):
         if self.kind == RECURSIVE_MUTEX and self._ismine():
             self.count += 1
             return space.w_True
+
         try:
-            # sets self.last_tid and increments self.count
-            # those steps need to be as close as possible to
-            # acquiring the semlock for self._ismine() to support
-            # multiple threads
             got = semlock_acquire(self, space, block, w_timeout)
         except OSError as e:
             raise wrap_oserror(space, e)
+
         if got:
+            self.last_tid = rthread.get_ident()
+            self.count += 1
             return space.w_True
         else:
             return space.w_False
@@ -526,31 +500,19 @@ class W_SemLock(W_Root):
                 return
 
         try:
-            # Note: a succesful semlock_release() must not release the GIL,
-            # otherwise there is a race condition on self.count
             semlock_release(self, space)
-            self.count -= 1
         except OSError as e:
             raise wrap_oserror(space, e)
 
+        self.count -= 1
 
     def after_fork(self):
         self.count = 0
 
-    @unwrap_spec(kind=int, maxvalue=int, name='text_or_none')
-    def rebuild(space, w_cls, w_handle, kind, maxvalue, name):
-        #
-        if sys_platform != 'win32' and name is not None:
-            # like CPython, in this case ignore 'w_handle'
-            try:
-                handle = reopen_semaphore(name)
-            except OSError as e:
-                raise wrap_oserror(space, e)
-        else:
-            handle = handle_w(space, w_handle)
-        #
+    @unwrap_spec(kind=int, maxvalue=int)
+    def rebuild(space, w_cls, w_handle, kind, maxvalue):
         self = space.allocate_instance(W_SemLock, w_cls)
-        self.__init__(space, handle, kind, maxvalue, name)
+        self.__init__(space, handle_w(space, w_handle), kind, maxvalue)
         return self
 
     def enter(self, space):
@@ -562,24 +524,21 @@ class W_SemLock(W_Root):
     def _finalize_(self):
         delete_semaphore(self.handle)
 
-@unwrap_spec(kind=int, value=int, maxvalue=int, name='text', unlink=int)
-def descr_new(space, w_subtype, kind, value, maxvalue, name, unlink):
+@unwrap_spec(kind=int, value=int, maxvalue=int)
+def descr_new(space, w_subtype, kind, value, maxvalue):
     if kind != RECURSIVE_MUTEX and kind != SEMAPHORE:
         raise oefmt(space.w_ValueError, "unrecognized kind")
 
     counter = space.fromcache(CounterState).getCount()
+    name = "/mp%d-%d" % (os.getpid(), counter)
 
     try:
         handle = create_semaphore(space, name, value, maxvalue)
-        if unlink:
-            sem_unlink(name)
-            name = None
     except OSError as e:
         raise wrap_oserror(space, e)
 
-
     self = space.allocate_instance(W_SemLock, w_subtype)
-    self.__init__(space, handle, kind, maxvalue, name)
+    self.__init__(space, handle, kind, maxvalue)
 
     return self
 
@@ -589,7 +548,6 @@ W_SemLock.typedef = TypeDef(
     kind = GetSetProperty(W_SemLock.kind_get),
     maxvalue = GetSetProperty(W_SemLock.maxvalue_get),
     handle = GetSetProperty(W_SemLock.handle_get),
-    name = GetSetProperty(W_SemLock.name_get),
     _count = interp2app(W_SemLock.get_count),
     _is_mine = interp2app(W_SemLock.is_mine),
     _is_zero = interp2app(W_SemLock.is_zero),
