@@ -69,8 +69,8 @@ def setup_directory_structure(space):
              foobar    = "found = 123",
              barbaz    = "other = 543")
     setuppkg("pkg.withoutall",
-             __init__  = "",
-             foobar    = "found = 123")
+             __init__  = "globals()[''] = 456",
+             foobar    = "found = 123\n")
     setuppkg("pkg.bogusall",
              __init__  = "__all__ = 42")
     setuppkg("pkg_r", inpkg = "import x.y")
@@ -108,7 +108,7 @@ def setup_directory_structure(space):
     # create compiled/x.py and a corresponding pyc file
     p = setuppkg("compiled", x = "x = 84")
     if conftest.option.runappdirect:
-        import marshal, stat, struct, os, imp
+        import marshal, stat, struct, imp
         code = py.code.Source(p.join("x.py").read()).compile()
         s3 = marshal.dumps(code)
         s2 = struct.pack("<i", os.stat(str(p.join("x.py")))[stat.ST_MTIME])
@@ -138,6 +138,15 @@ def setup_directory_structure(space):
         pass
     p.join('x.py').rename(p.join('x.pyw'))
 
+    if hasattr(p, "mksymlinkto"):
+        p = root.join("devnullpkg")
+        p.ensure(dir=True)
+        p.join("__init__.py").mksymlinkto(os.devnull)
+
+    p = root.join("onlypyw")
+    p.ensure(dir=True)
+    p.join("__init__.pyw")
+
     return str(root)
 
 def _load_source_module(space, w_modname, w_mod, *args, **kwds):
@@ -289,6 +298,13 @@ class AppTestImport:
         assert ambig == sys.modules.get('ambig')
         assert hasattr(ambig,'imapackage')
 
+    def test_trailing_dot(self):
+        # bug-for-bug compatibility with CPython
+        import sys
+        __import__('pkg.pkg1.')
+        assert 'pkg.pkg1' in sys.modules
+        assert 'pkg.pkg1.' not in sys.modules
+
     def test_from_a(self):
         import sys
         from a import imamodule
@@ -351,6 +367,21 @@ class AppTestImport:
         o = __import__('sys', [], [], ['']) # CPython accepts this
         assert sys == o
 
+    def test_import_fromlist_must_not_contain_unicodes(self):
+        import sys
+        ver = sys.version_info
+        exc = raises(TypeError, __import__, 'encodings', None, None, [u'xxx'])
+        if ver > (2, 7, 12):
+            assert 'must be str' in exc.value.message
+        exc = raises(TypeError, __import__, 'encodings', None, None, [123])
+        if ver > (2, 7, 12):
+            assert 'must be str' in exc.value.message
+        # issue 2524
+        raises(ImportError, __import__, 'xxxbadmodule', fromlist=[u'xx'])
+        mod = __import__('collections', fromlist=[u'defaultdict'])
+        assert mod is not None
+
+
     def test_import_relative_back_to_absolute2(self):
         from pkg import abs_x_y
         import sys
@@ -721,6 +752,12 @@ class AppTestImport:
             exec "from pkg.withoutall import *" in d
             assert d["foobar"].found == 123
 
+    def test_import_star_empty_string(self):
+        for case in ["not-imported-yet", "already-imported"]:
+            d = {}
+            exec "from pkg.withoutall import *" in d
+            assert "" in d
+
     def test_import_star_with_bogus___all__(self):
         for case in ["not-imported-yet", "already-imported"]:
             try:
@@ -792,6 +829,15 @@ class AppTestImport:
             reload(sys)
         assert not output
 
+    def test_dir_with_only_pyw(self):
+        def imp():
+            import onlypyw
+        raises(ImportError, imp)
+
+    @pytest.mark.skipif(not hasattr(py.path.local, "mksymlinkto"), reason="requires symlinks")
+    def test_dev_null_init_file(self):
+        import devnullpkg
+
 
 class TestAbi:
     def test_abi_tag(self):
