@@ -15,6 +15,8 @@ class FakeArrayImplementation(BaseConcreteArray):
         self._base = base
         self.dtype = base.get_dtype()
         self.shape = [base.get_size()]
+        self.storage = self._base.implementation.storage
+        self.order = base.get_order()
 
     def base(self):
         return self._base
@@ -38,20 +40,20 @@ class W_FlatIterator(W_NDimArray):
         self.implementation = FakeArrayImplementation(self.base)
 
     def descr_base(self, space):
-        return space.wrap(self.base)
+        return self.base
 
     def descr_index(self, space):
-        return space.wrap(self.state.index)
+        return space.newint(self.state.index)
 
     def descr_coords(self, space):
         coords = self.iter.indices(self.state)
-        return space.newtuple([space.wrap(c) for c in coords])
+        return space.newtuple([space.newint(c) for c in coords])
 
     def descr_iter(self):
         return self
 
     def descr_len(self, space):
-        return space.wrap(self.iter.size)
+        return space.newint(self.iter.size)
 
     def descr_next(self, space):
         if self.iter.done(self.state):
@@ -96,6 +98,8 @@ class W_FlatIterator(W_NDimArray):
         finally:
             self.iter.reset(self.state, mutate=True)
 
+    def descr___array_wrap__(self, space, obj, w_context=None):
+        return obj
 
 W_FlatIterator.typedef = TypeDef("numpy.flatiter",
     base = GetSetProperty(W_FlatIterator.descr_base),
@@ -115,4 +119,5 @@ W_FlatIterator.typedef = TypeDef("numpy.flatiter",
     __le__ = interp2app(W_FlatIterator.descr_le),
     __gt__ = interp2app(W_FlatIterator.descr_gt),
     __ge__ = interp2app(W_FlatIterator.descr_ge),
+    __array_wrap__ = interp2app(W_NDimArray.descr___array_wrap__),
 )
