@@ -31,7 +31,6 @@ class AppTestMMap:
         assert isinstance(mmap.ACCESS_READ, int)
         assert isinstance(mmap.ACCESS_WRITE, int)
         assert isinstance(mmap.ACCESS_COPY, int)
-        assert isinstance(mmap.ACCESS_DEFAULT, int)
         if os.name == "posix":
             assert isinstance(mmap.MAP_ANON, int)
             assert isinstance(mmap.MAP_ANONYMOUS, int)
@@ -41,7 +40,9 @@ class AppTestMMap:
             assert isinstance(mmap.PROT_READ, int)
             assert isinstance(mmap.PROT_WRITE, int)
 
-        assert mmap.error is OSError
+        assert 'mmap.error' in str(mmap.error)
+        assert mmap.error is not EnvironmentError
+        assert issubclass(mmap.error, EnvironmentError)
 
     def test_args(self):
         from mmap import mmap
@@ -56,7 +57,7 @@ class AppTestMMap:
             raises(TypeError, mmap, 0, 1, 2, 3, "foo", 5)
             raises(TypeError, mmap, 0, 1, foo="foo")
             raises((TypeError, OverflowError), mmap, 0, -1)
-            raises(OverflowError, mmap, 0, sys.maxsize ** 3)
+            raises(OverflowError, mmap, 0, sys.maxint ** 3)
             raises(ValueError, mmap, 0, 1, flags=2, access=3)
             raises(ValueError, mmap, 0, 1, access=123)
         elif os.name == "nt":
@@ -78,30 +79,29 @@ class AppTestMMap:
             skip("Only Unix checks file size")
 
         from mmap import mmap
-        f = open(self.tmpname + "a", "wb+")
+        f = open(self.tmpname + "a", "w+")
 
-        f.write(b"c")
-        ret = f.flush()
-        assert ret is None
+        f.write("c")
+        f.flush()
         raises(ValueError, mmap, f.fileno(), 123)
         f.close()
 
     def test_create(self):
         from mmap import mmap
-        f = open(self.tmpname + "b", "wb+")
+        f = open(self.tmpname + "b", "w+")
 
-        f.write(b"c")
+        f.write("c")
         f.flush()
         m = mmap(f.fileno(), 1)
-        assert m.read(99) == b"c"
+        assert m.read(99) == "c"
 
         f.close()
 
     def test_close(self):
         from mmap import mmap
-        f = open(self.tmpname + "c", "wb+")
+        f = open(self.tmpname + "c", "w+")
 
-        f.write(b"c")
+        f.write("c")
         f.flush()
         m = mmap(f.fileno(), 1)
         m.close()
@@ -109,12 +109,12 @@ class AppTestMMap:
 
     def test_read_byte(self):
         from mmap import mmap
-        f = open(self.tmpname + "d", "wb+")
+        f = open(self.tmpname + "d", "w+")
 
-        f.write(b"c")
+        f.write("c")
         f.flush()
         m = mmap(f.fileno(), 1)
-        assert m.read_byte() == ord(b"c")
+        assert m.read_byte() == "c"
         raises(ValueError, m.read_byte)
         m.close()
         f.close()
@@ -122,121 +122,127 @@ class AppTestMMap:
     def test_readline(self):
         from mmap import mmap
         import os
-        with open(self.tmpname + "e", "wb+") as f:
-            f.write(b"foo\n")
-            f.flush()
-            with mmap(f.fileno(), 4) as m:
-                result = m.readline()
-                assert result == b"foo\n"
-                assert m.readline() == b""
+        f = open(self.tmpname + "e", "w+")
+
+        f.write("foo\n")
+        f.flush()
+        m = mmap(f.fileno(), 4)
+        if os.name == "nt":
+            # windows replaces \n with \r. it's time to change to \n only MS!
+            assert m.readline() == "foo\r"
+        elif os.name == "posix":
+            assert m.readline() == "foo\n"
+        assert m.readline() == ""
+        m.close()
+        f.close()
 
     def test_read(self):
         from mmap import mmap
-        f = open(self.tmpname + "f", "wb+")
+        f = open(self.tmpname + "f", "w+")
 
-        f.write(b"foobar")
+        f.write("foobar")
         f.flush()
         m = mmap(f.fileno(), 6)
-        raises(TypeError, m.read, b"foo")
-        assert m.read(1) == b"f"
-        assert m.read(6) == b"oobar"
-        assert m.read(1) == b""
+        raises(TypeError, m.read, "foo")
+        assert m.read(1) == "f"
+        assert m.read(6) == "oobar"
+        assert m.read(1) == ""
         m.close()
         f.close()
 
     def test_find(self):
         from mmap import mmap
-        f = open(self.tmpname + "g", "wb+")
+        f = open(self.tmpname + "g", "w+")
 
-        f.write(b"foobar\0")
+        f.write("foobar\0")
         f.flush()
         m = mmap(f.fileno(), 7)
         raises(TypeError, m.find, 123)
-        raises(TypeError, m.find, b"foo", b"baz")
-        assert m.find(b"b") == 3
-        assert m.find(b"z") == -1
-        assert m.find(b"o", 5) == -1
-        assert m.find(b"ob") == 2
-        assert m.find(b"\0") == 6
-        assert m.find(b"ob", 1) == 2
-        assert m.find(b"ob", 2) == 2
-        assert m.find(b"ob", 3) == -1
-        assert m.find(b"ob", -4) == -1
-        assert m.find(b"ob", -5) == 2
-        assert m.find(b"ob", -999999999) == 2
-        assert m.find(b"ob", 1, 3) == -1
-        assert m.find(b"ob", 1, 4) == 2
-        assert m.find(b"ob", 1, 999999999) == 2
-        assert m.find(b"ob", 1, 0) == -1
-        assert m.find(b"ob", 1, -1) == 2
-        assert m.find(b"ob", 1, -3) == 2
-        assert m.find(b"ob", 1, -4) == -1
+        raises(TypeError, m.find, "foo", "baz")
+        assert m.find("b") == 3
+        assert m.find("z") == -1
+        assert m.find("o", 5) == -1
+        assert m.find("ob") == 2
+        assert m.find("\0") == 6
+        assert m.find("ob", 1) == 2
+        assert m.find("ob", 2) == 2
+        assert m.find("ob", 3) == -1
+        assert m.find("ob", -4) == -1
+        assert m.find("ob", -5) == 2
+        assert m.find("ob", -999999999) == 2
+        assert m.find("ob", 1, 3) == -1
+        assert m.find("ob", 1, 4) == 2
+        assert m.find("ob", 1, 999999999) == 2
+        assert m.find("ob", 1, 0) == -1
+        assert m.find("ob", 1, -1) == 2
+        assert m.find("ob", 1, -3) == 2
+        assert m.find("ob", 1, -4) == -1
         #
         data = m.read(2)
-        assert data == b"fo"
-        assert m.find(b"o") == 2
-        assert m.find(b"oo") == -1
-        assert m.find(b"o", 0) == 1
+        assert data == "fo"
+        assert m.find("o") == 2
+        assert m.find("oo") == -1
+        assert m.find("o", 0) == 1
         m.close()
         f.close()
 
     def test_rfind(self):
         from mmap import mmap
-        f = open(self.tmpname + "g", "wb+")
+        f = open(self.tmpname + "g", "w+")
 
-        f.write(b"foobarfoobar\0")
+        f.write("foobarfoobar\0")
         f.flush()
         m = mmap(f.fileno(), 13)
         raises(TypeError, m.rfind, 123)
-        raises(TypeError, m.rfind, b"foo", b"baz")
-        assert m.rfind(b"b") == 9
-        assert m.rfind(b"z") == -1
-        assert m.rfind(b"o", 11) == -1
-        assert m.rfind(b"ob") == 8
-        assert m.rfind(b"\0") == 12
-        assert m.rfind(b"ob", 7) == 8
-        assert m.rfind(b"ob", 8) == 8
-        assert m.rfind(b"ob", 9) == -1
-        assert m.rfind(b"ob", -4) == -1
-        assert m.rfind(b"ob", -5) == 8
-        assert m.rfind(b"ob", -999999999) == 8
-        assert m.rfind(b"ob", 1, 3) == -1
-        assert m.rfind(b"ob", 1, 4) == 2
-        assert m.rfind(b"ob", 1, 999999999) == 8
-        assert m.rfind(b"ob", 1, 0) == -1
-        assert m.rfind(b"ob", 1, -1) == 8
-        assert m.rfind(b"ob", 1, -3) == 8
-        assert m.rfind(b"ob", 1, -4) == 2
+        raises(TypeError, m.rfind, "foo", "baz")
+        assert m.rfind("b") == 9
+        assert m.rfind("z") == -1
+        assert m.rfind("o", 11) == -1
+        assert m.rfind("ob") == 8
+        assert m.rfind("\0") == 12
+        assert m.rfind("ob", 7) == 8
+        assert m.rfind("ob", 8) == 8
+        assert m.rfind("ob", 9) == -1
+        assert m.rfind("ob", -4) == -1
+        assert m.rfind("ob", -5) == 8
+        assert m.rfind("ob", -999999999) == 8
+        assert m.rfind("ob", 1, 3) == -1
+        assert m.rfind("ob", 1, 4) == 2
+        assert m.rfind("ob", 1, 999999999) == 8
+        assert m.rfind("ob", 1, 0) == -1
+        assert m.rfind("ob", 1, -1) == 8
+        assert m.rfind("ob", 1, -3) == 8
+        assert m.rfind("ob", 1, -4) == 2
         #
         data = m.read(8)
-        assert data == b"foobarfo"
-        assert m.rfind(b"o") == 8
-        assert m.rfind(b"oo") == -1
-        assert m.rfind(b"o", 0) == 8
+        assert data == "foobarfo"
+        assert m.rfind("o") == 8
+        assert m.rfind("oo") == -1
+        assert m.rfind("o", 0) == 8
         m.close()
         f.close()
 
     def test_is_modifiable(self):
         import mmap
-        f = open(self.tmpname + "h", "wb+")
+        f = open(self.tmpname + "h", "w+")
 
-        f.write(b"foobar")
+        f.write("foobar")
         f.flush()
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_READ)
-        raises(TypeError, m.write, b'x')
+        raises(TypeError, m.write, 'x')
         raises(TypeError, m.resize, 7)
         m.close()
         f.close()
 
     def test_seek(self):
         from mmap import mmap
-        f = open(self.tmpname + "i", "wb+")
+        f = open(self.tmpname + "i", "w+")
 
-        f.write(b"foobar")
+        f.write("foobar")
         f.flush()
         m = mmap(f.fileno(), 6)
-        raises(TypeError, m.seek, b"foo")
-        raises(TypeError, m.seek, 0, b"foo")
+        raises(TypeError, m.seek, "foo")
+        raises(TypeError, m.seek, 0, "foo")
         raises(ValueError, m.seek, -1, 0)
         raises(ValueError, m.seek, -1, 1)
         raises(ValueError, m.seek, -7, 2)
@@ -255,41 +261,41 @@ class AppTestMMap:
 
     def test_write(self):
         import mmap
-        f = open(self.tmpname + "j", "wb+")
+        f = open(self.tmpname + "j", "w+")
 
-        f.write(b"foobar")
+        f.write("foobar")
         f.flush()
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_READ)
-        raises(TypeError, m.write, b"foo")
+        raises(TypeError, m.write, "foo")
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_WRITE)
         raises(TypeError, m.write, 123)
-        raises(ValueError, m.write, b"c"*10)
-        assert m.write(b"ciao\n") == 5
+        raises(ValueError, m.write, "c"*10)
+        m.write("ciao\n")
         m.seek(0)
-        assert m.read(6) == b"ciao\nr"
+        assert m.read(6) == "ciao\nr"
         m.close()
 
     def test_write_byte(self):
         import mmap
-        f = open(self.tmpname + "k", "wb+")
+        f = open(self.tmpname + "k", "w+")
 
-        f.write(b"foobar")
+        f.write("foobar")
         f.flush()
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_READ)
-        raises(TypeError, m.write_byte, b"f")
+        raises(TypeError, m.write_byte, "f")
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_WRITE)
-        raises(TypeError, m.write_byte, "a")
-        raises(TypeError, m.write_byte, b"a")
-        m.write_byte(ord("x"))
+        raises(TypeError, m.write_byte, 123)
+        raises(TypeError, m.write_byte, "ab")
+        m.write_byte("x")
         m.seek(0)
-        assert m.read(6) == b"xoobar"
+        assert m.read(6) == "xoobar"
         m.close()
 
     def test_size(self):
         from mmap import mmap
-        f = open(self.tmpname + "l", "wb+")
+        f = open(self.tmpname + "l", "w+")
 
-        f.write(b"foobar")
+        f.write("foobar")
         f.flush()
         m = mmap(f.fileno(), 5)
         assert m.size() == 6 # size of the underline file, not the mmap
@@ -298,9 +304,9 @@ class AppTestMMap:
 
     def test_tell(self):
         from mmap import mmap
-        f = open(self.tmpname + "m", "wb+")
+        f = open(self.tmpname + "m", "w+")
 
-        f.write(b"c")
+        f.write("c")
         f.flush()
         m = mmap(f.fileno(), 1)
         assert m.tell() >= 0
@@ -309,13 +315,13 @@ class AppTestMMap:
 
     def test_flush(self):
         from mmap import mmap
-        f = open(self.tmpname + "n", "wb+")
+        f = open(self.tmpname + "n", "w+")
 
-        f.write(b"foobar")
+        f.write("foobar")
         f.flush()
         m = mmap(f.fileno(), 6)
         raises(TypeError, m.flush, 1, 2, 3)
-        raises(TypeError, m.flush, 1, b"a")
+        raises(TypeError, m.flush, 1, "a")
         raises(ValueError, m.flush, 0, 99)
         m.flush()    # return value is a bit meaningless, platform-dependent
         m.close()
@@ -325,28 +331,28 @@ class AppTestMMap:
         import mmap
 
         with open(self.tmpname, "wb") as f:
-            f.write(115699 * b'm')
+            f.write(115699 * 'm')
         with open(self.tmpname, "w+b") as f:
             raises(ValueError, mmap.mmap, f.fileno(), 0, offset=2147418112)
 
     def test_move(self):
         import mmap
-        f = open(self.tmpname + "o", "wb+")
+        f = open(self.tmpname + "o", "w+")
 
-        f.write(b"foobar")
+        f.write("foobar")
         f.flush()
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_READ)
         raises(TypeError, m.move, 1)
-        raises(TypeError, m.move, 1, b"foo", 2)
+        raises(TypeError, m.move, 1, "foo", 2)
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_WRITE)
         raises(ValueError, m.move, 7, 1, 2)
         raises(ValueError, m.move, 1, 7, 2)
         m.move(1, 3, 3)
-        assert m.read(6) == b"fbarar"
+        assert m.read(6) == "fbarar"
         m.seek(0)
         m.move(1, 3, 2)
         a = m.read(6)
-        assert a == b"frarar"
+        assert a == "frarar"
         m.close()
         f.close()
 
@@ -358,8 +364,8 @@ class AppTestMMap:
         import mmap
         import os
 
-        f = open(self.tmpname + "p", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "p", "w+")
+        f.write("foobar")
         f.flush()
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_READ)
         raises(TypeError, m.resize, 1)
@@ -382,8 +388,8 @@ class AppTestMMap:
         import mmap
         import os
 
-        f = open(self.tmpname + "p", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "p", "w+")
+        f.write("foobar")
         f.flush()
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_READ)
         raises(TypeError, m.resize, 1)
@@ -399,8 +405,8 @@ class AppTestMMap:
     def test_len(self):
         from mmap import mmap
 
-        f = open(self.tmpname + "q", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "q", "w+")
+        f.write("foobar")
         f.flush()
 
         m = mmap(f.fileno(), 6)
@@ -411,78 +417,70 @@ class AppTestMMap:
     def test_get_item(self):
         from mmap import mmap
 
-        f = open(self.tmpname + "r", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "r", "w+")
+        f.write("foobar")
         f.flush()
 
         m = mmap(f.fileno(), 6)
-        fn = lambda: m[b"foo"]
+        fn = lambda: m["foo"]
         raises(TypeError, fn)
         fn = lambda: m[-7]
         raises(IndexError, fn)
-        assert m[0] == ord('f')
-        assert m[-1] == ord('r')
-        assert m[1::2] == b'obr'
-        assert m[4:1:-2] == b'ao'
+        assert m[0] == 'f'
+        assert m[-1] == 'r'
+        assert m[1::2] == 'obr'
+        assert m[4:1:-2] == 'ao'
         m.close()
         f.close()
 
-    def test_get_crash(self):
-        import sys
-        from mmap import mmap
-        s = b'hallo!!!'
-        m = mmap(-1, len(s))
-        m[:] = s
-        assert m[1:None:sys.maxsize] == b'a'
-        m.close()
-
     def test_set_item(self):
         import mmap
 
-        f = open(self.tmpname + "s", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "s", "w+")
+        f.write("foobar")
         f.flush()
 
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_READ)
-        def fn(): m[1] = b'a'
+        def fn(): m[1] = 'a'
         raises(TypeError, fn)
         m = mmap.mmap(f.fileno(), 6, access=mmap.ACCESS_WRITE)
-        def fn(): m[b"foo"] = b'a'
+        def fn(): m["foo"] = 'a'
         raises(TypeError, fn)
-        def fn(): m[-7] = b'a'
+        def fn(): m[-7] = 'a'
         raises(IndexError, fn)
-        def fn(): m[1:3] = 'xx'
+        def fn(): m[0] = 'ab'
+        raises((IndexError, ValueError), fn)     # IndexError is in CPython,
+                                                 # but doesn't make much sense
+        def fn(): m[1:3] = u'xx'
         raises((IndexError, TypeError), fn)      # IndexError is in CPython,
                                                  # but doesn't make much sense
-        def fn(): m[1:4] = b"zz"
+        def fn(): m[1:4] = "zz"
         raises((IndexError, ValueError), fn)
-        def fn(): m[1:6] = b"z" * 6
+        def fn(): m[1:6] = "z" * 6
         raises((IndexError, ValueError), fn)
-        def fn(): m[:2] = b"z" * 5
+        def fn(): m[:2] = "z" * 5
         raises((IndexError, ValueError), fn)
-        def fn(): m[0] = 256
-        raises(ValueError, fn)
-        m[1:3] = b'xx'
-        assert m.read(6) == b"fxxbar"
-        m[0] = ord('x')
-        assert m[0] == ord('x')
-        m[-6] = ord('y')
-        m[3:6:2] = b'BR'
+        m[1:3] = 'xx'
+        assert m.read(6) == "fxxbar"
+        m[0] = 'x'
+        assert m[0] == 'x'
+        m[-6] = 'y'
+        m[3:6:2] = 'BR'
         m.seek(0)
         data = m.read(6)
-        assert data == b"yxxBaR"
+        assert data == "yxxBaR"
         m.close()
         f.close()
 
     def test_del_item(self):
         from mmap import mmap
 
-        f = open(self.tmpname + "t", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "t", "w+")
+        f.write("foobar")
         f.flush()
 
         m = mmap(f.fileno(), 6)
-        def fn(): del m[b"foo"]
+        def fn(): del m["foo"]
         raises(TypeError, fn)
         def fn(): del m[1:3]
         raises(TypeError, fn)
@@ -494,8 +492,8 @@ class AppTestMMap:
     def test_concatenation(self):
         from mmap import mmap
 
-        f = open(self.tmpname + "u", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "u", "w+")
+        f.write("foobar")
         f.flush()
 
         m = mmap(f.fileno(), 6)
@@ -511,8 +509,8 @@ class AppTestMMap:
     def test_repeatition(self):
         from mmap import mmap
 
-        f = open(self.tmpname + "v", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "v", "w+")
+        f.write("foobar")
         f.flush()
 
         m = mmap(f.fileno(), 6)
@@ -528,70 +526,101 @@ class AppTestMMap:
     def test_slicing(self):
         from mmap import mmap
 
-        f = open(self.tmpname + "v", "wb+")
-        f.write(b"foobar")
+        f = open(self.tmpname + "v", "w+")
+        f.write("foobar")
         f.flush()
 
         f.seek(0)
         m = mmap(f.fileno(), 6)
-        assert m[-3:7] == b"bar"
+        assert m[-3:7] == "bar"
+        assert m.__getslice__(-3, 7) == "foobar"
+        m.__setslice__(2, 4, "zz")
+        assert m.__getslice__(-3, 7) == "fozzar"
+        raises(TypeError, m.__getslice__, "abc", 2)
+        raises(IndexError, m.__setslice__, 2, 4, None)
 
-        assert m[1:0:1] == b""
+        assert m[1:0:1] == ""
 
         f.close()
 
+    def test_sequence_type(self):
+        from mmap import mmap
+        f = open(self.tmpname + "x", "w+")
+        f.write("foobar")
+        f.flush()
+        m = mmap(f.fileno(), 6)
+        import operator
+        assert operator.isSequenceType(m)
+        assert not operator.isMappingType(m)
+
+    def test_buffer(self):
+        from mmap import mmap
+        f = open(self.tmpname + "y", "w+")
+        f.write("foobar")
+        f.flush()
+        m = mmap(f.fileno(), 6)
+        b = buffer(m)
+        assert len(b) == 6
+        assert b[3] == "b"
+        assert b[:] == "foobar"
+        m.close()
+        f.close()
+
+    def test_buffer_write(self):
+        from mmap import mmap
+        f = open(self.tmpname + "y", "w+")
+        f.write("foobar")
+        f.flush()
+        m = mmap(f.fileno(), 6)
+        m[5] = '?'
+        b = buffer(m)
+        exc = raises(TypeError, 'b[:3] = "FOO"')
+        assert str(exc.value) == "buffer is read-only"
+        m.close()
+        f.seek(0)
+        got = f.read()
+        assert got == "fooba?"
+        f.close()
+
     def test_memoryview(self):
         from mmap import mmap
-        filename = self.tmpname + "y"
-        with open(filename, "bw+") as f:
-            f.write(b"foobar")
-            f.flush()
-            with mmap(f.fileno(), 6) as m:
-                b = memoryview(m)
-                assert len(b) == 6
-                assert b.readonly is False
-                assert b[3] == ord(b"b")
-                assert b[:] == b"foobar"
-                del b  # For CPython: "exported pointers exist"
-        try:
-            from mmap import PROT_READ
-        except ImportError:
-            skip('no PROT_READ') 
-        with open(filename, "rb") as f:
-            m = mmap(f.fileno(), 6, prot=PROT_READ)
-            b = memoryview(m)
-            assert b.readonly is True
-            assert b[:] == b"foobar"
-            del b
-            m.close()
+        f = open(self.tmpname + "y", "w+")
+        f.write("foobar")
+        f.flush()
+        m = mmap(f.fileno(), 6)
+        m[5] = '?'
+        exc = raises(TypeError, memoryview, m)
+        assert 'buffer interface' in str(exc.value)
+        m.close()
+        f.close()
 
     def test_offset(self):
         from mmap import mmap, ALLOCATIONGRANULARITY
-        filename = self.tmpname + "y"
-        with open(filename, "wb+") as f:
-            f.write(b"foobar" * ALLOCATIONGRANULARITY)
-            f.flush()
-            size = ALLOCATIONGRANULARITY
-            offset = 2 * ALLOCATIONGRANULARITY
-            m = mmap(f.fileno(), size, offset=offset)
-            assert m[:] == (b"foobar" * ALLOCATIONGRANULARITY)[offset:offset+size]
-            assert len(m) == size
-            m.close()
+        f = open(self.tmpname + "y", "w+")
+        f.write("foobar" * ALLOCATIONGRANULARITY)
+        f.flush()
+        size = ALLOCATIONGRANULARITY
+        offset = 2 * ALLOCATIONGRANULARITY
+        m = mmap(f.fileno(), size, offset=offset)
+        assert m[:] == ("foobar" * ALLOCATIONGRANULARITY)[offset:offset+size]
+        assert len(m) == size
+        m.close()
+        f.close()
 
     def test_offset_more(self):
         from mmap import mmap, ALLOCATIONGRANULARITY
 
         with open(self.tmpname, "w+b") as f:
             halfsize = ALLOCATIONGRANULARITY
-            f.write(b"\0" * halfsize)
-            f.write(b"foo")
-            f.write(b"\0" * (halfsize - 3))
+            f.write("\0" * halfsize)
+            f.write("foo")
+            f.write("\0" * (halfsize - 3))
             m = mmap(f.fileno(), 0)
             m.close()
 
         with open(self.tmpname, "r+b") as f:
             m = mmap(f.fileno(), halfsize, offset=halfsize)
-            assert m[0:3] == b"foo"
+            assert m[0:3] == "foo"
 
         try:
             m.resize(512)
@@ -600,7 +629,7 @@ class AppTestMMap:
         else:
             assert len(m) == 512
             raises(ValueError, m.seek, 513, 0)
-            assert m[0:3] == b"foo"
+            assert m[0:3] == "foo"
             with open(self.tmpname) as f:
                 f.seek(0, 2)
                 assert f.tell() == halfsize + 512
@@ -616,7 +645,7 @@ class AppTestMMap:
 
         with open(self.tmpname, "w+b") as f:
             f.seek(size)
-            f.write(b"A")
+            f.write("A")
             f.flush()
         with open(self.tmpname, 'rb') as f2:
             f2.seek(size)
@@ -625,7 +654,7 @@ class AppTestMMap:
             m = mmap.mmap(f2.fileno(), 0, offset=0x140000000,
                           access=mmap.ACCESS_READ)
             try:
-                assert m[0xFFFFFFF] == ord('A')
+                assert m[0xFFFFFFF] == b'A'
             finally:
                 m.close()
     test_large_offset.is_large = True
@@ -639,7 +668,7 @@ class AppTestMMap:
 
         with open(self.tmpname, "w+b") as f:
             f.seek(size)
-            f.write(b" ")
+            f.write(" ")
             f.flush()
             m = mmap.mmap(f.fileno(), 0x10000, access=mmap.ACCESS_READ)
             try:
@@ -648,12 +677,6 @@ class AppTestMMap:
                 m.close()
     test_large_filesize.is_large = True
 
-    def test_context_manager(self):
-        import mmap
-        with mmap.mmap(-1, 10) as m:
-            assert not m.closed
-        assert m.closed
-
     def test_all(self):
         # this is a global test, ported from test_mmap.py
         import mmap
@@ -663,30 +686,30 @@ class AppTestMMap:
 
         filename = self.tmpname + "w"
 
-        f = open(filename, "wb+")
+        f = open(filename, "w+")
 
         # write 2 pages worth of data to the file
-        f.write(b'\0' * PAGESIZE)
-        f.write(b'foo')
-        f.write(b'\0' * (PAGESIZE - 3))
+        f.write('\0' * PAGESIZE)
+        f.write('foo')
+        f.write('\0' * (PAGESIZE - 3))
         f.flush()
         m = mmap.mmap(f.fileno(), 2 * PAGESIZE)
         f.close()
 
         # sanity checks
-        assert m.find(b"foo") == PAGESIZE
+        assert m.find("foo") == PAGESIZE
         assert len(m) == 2 * PAGESIZE
-        assert m[0] == 0
-        assert m[0:3] == b'\0\0\0'
+        assert m[0] == '\0'
+        assert m[0:3] == '\0\0\0'
 
         # modify the file's content
-        m[0] = ord('3')
-        m[PAGESIZE+3:PAGESIZE+3+3] = b'bar'
+        m[0] = '3'
+        m[PAGESIZE+3:PAGESIZE+3+3] = 'bar'
 
         # check that the modification worked
-        assert m[0] == ord('3')
-        assert m[0:3] == b'3\0\0'
-        assert m[PAGESIZE-1:PAGESIZE+7] == b'\0foobar\0'
+        assert m[0] == '3'
+        assert m[0:3] == '3\0\0'
+        assert m[PAGESIZE-1:PAGESIZE+7] == '\0foobar\0'
 
         m.flush()
 
@@ -722,22 +745,22 @@ class AppTestMMap:
         # test access=ACCESS_READ
         mapsize = 10
         f = open(filename, "wb")
-        f.write(b"a" * mapsize)
+        f.write("a" * mapsize)
         f.close()
         f = open(filename, "rb")
         m = mmap.mmap(f.fileno(), mapsize, access=mmap.ACCESS_READ)
-        assert m[:] == b'a' * mapsize
-        def f(m): m[:] = b'b' * mapsize
+        assert m[:] == 'a' * mapsize
+        def f(m): m[:] = 'b' * mapsize
         raises(TypeError, f, m)
-        def fn(): m[0] = b'b'
+        def fn(): m[0] = 'b'
         raises(TypeError, fn)
-        def fn(m): m.seek(0, 0); m.write(b"abc")
+        def fn(m): m.seek(0, 0); m.write("abc")
         raises(TypeError, fn, m)
-        def fn(m): m.seek(0, 0); m.write_byte(b"d")
+        def fn(m): m.seek(0, 0); m.write_byte("d")
         raises(TypeError, fn, m)
         if not (("darwin" in sys.platform) or ("freebsd" in sys.platform)):
             raises(TypeError, m.resize, 2 * mapsize)
-            assert open(filename, "rb").read() == b'a' * mapsize
+            assert open(filename, "rb").read() == 'a' * mapsize
 
         # opening with size too big
         f = open(filename, "r+b")
@@ -756,27 +779,27 @@ class AppTestMMap:
         # test access=ACCESS_WRITE"
         f = open(filename, "r+b")
         m = mmap.mmap(f.fileno(), mapsize, access=mmap.ACCESS_WRITE)
-        m.write(b'c' * mapsize)
+        m.write('c' * mapsize)
         m.seek(0)
         data = m.read(mapsize)
-        assert data == b'c' * mapsize
+        assert data == 'c' * mapsize
         m.flush()
         m.close()
         f.close()
         f = open(filename, 'rb')
         stuff = f.read()
         f.close()
-        assert stuff == b'c' * mapsize
+        assert stuff == 'c' * mapsize
 
         # test access=ACCESS_COPY
         f = open(filename, "r+b")
         m = mmap.mmap(f.fileno(), mapsize, access=mmap.ACCESS_COPY)
-        m.write(b'd' * mapsize)
+        m.write('d' * mapsize)
         m.seek(0)
         data = m.read(mapsize)
-        assert data == b'd' * mapsize
+        assert data == 'd' * mapsize
         m.flush()
-        assert open(filename, "rb").read() == b'c' * mapsize
+        assert open(filename, "rb").read() == 'c' * mapsize
         if not (("darwin" in sys.platform) or ("freebsd" in sys.platform)):
             raises(TypeError, m.resize, 2 * mapsize)
         m.close()
@@ -800,8 +823,8 @@ class AppTestMMap:
 
         # do a tougher .find() test.  SF bug 515943 pointed out that, in 2.2,
         # searching for data with embedded \0 bytes didn't work.
-        f = open(filename, 'wb+')
-        data = b'aabaac\x00deef\x00\x00aa\x00'
+        f = open(filename, 'w+')
+        data = 'aabaac\x00deef\x00\x00aa\x00'
         n = len(data)
         f.write(data)
         f.flush()
@@ -812,27 +835,27 @@ class AppTestMMap:
             for finish in range(start, n + 1):
                 sl = data[start:finish]
                 assert m.find(sl) == data.find(sl)
-                assert m.find(sl + b'x') ==  -1
+                assert m.find(sl + 'x') ==  -1
         m.close()
 
         # test mapping of entire file by passing 0 for map length
-        f = open(filename, "wb+")
-        f.write(2**16 * b'm')
+        f = open(filename, "w+")
+        f.write(2**16 * 'm')
         f.close()
         f = open(filename, "rb+")
         m = mmap.mmap(f.fileno(), 0)
         assert len(m) == 2**16
-        assert m.read(2**16) == 2**16 * b"m"
+        assert m.read(2**16) == 2**16 * "m"
         m.close()
         f.close()
 
         # make move works everywhere (64-bit format problem earlier)
-        f = open(filename, 'wb+')
-        f.write(b"ABCDEabcde")
+        f = open(filename, 'w+')
+        f.write("ABCDEabcde")
         f.flush()
         m = mmap.mmap(f.fileno(), 10)
         m.move(5, 0, 5)
-        assert m.read(10) == b"ABCDEABCDE"
+        assert m.read(10) == "ABCDEABCDE"
         m.close()
         f.close()
 
@@ -850,15 +873,6 @@ class AppTestMMap:
             except BaseException as e:
                 assert False, "unexpected exception: " + str(e)
 
-    def test_read_all(self):
-        from mmap import mmap
-        f = open(self.tmpname + "f", "wb+")
-        f.write(b"foobar")
-        f.flush()
-
-        m = mmap(f.fileno(), 6)
-        assert m.read(None) == b"foobar"
-
     def test_resize_past_pos(self):
         import os, mmap, sys
         if os.name == "nt":
@@ -872,42 +886,11 @@ class AppTestMMap:
         except SystemError:
             skip("resizing not supported")
         assert m.tell() == 5000
-        assert m.read(14) == b''
-        assert m.read(-1) == b''
+        assert m.read(14) == ''
+        assert m.read(-1) == ''
         raises(ValueError, m.read_byte)
-        assert m.readline() == b''
-        raises(ValueError, m.write_byte, ord(b'b'))
-        raises(ValueError, m.write, b'abc')
+        assert m.readline() == ''
+        raises(ValueError, m.write_byte, 'b')
+        raises(ValueError, m.write, 'abc')
         assert m.tell() == 5000
         m.close()
-
-    def test_iter_yields_bytes(self):
-        # issue 3282: inconsistency in Python 3
-        from mmap import mmap
-        f = open(self.tmpname + "iter", "wb+")
-        f.write(b"AB")
-        f.flush()
-
-        m = mmap(f.fileno(), 2)
-        assert [m[0], m[1]] == [65, 66]
-        assert list(m) == [b"A", b"B"]
-        assert list(iter(m)) == [b"A", b"B"]
-        assert list(reversed(m)) == [b"B", b"A"]
-        assert list(enumerate(m)) == [(0, b"A"), (1, b"B")]
-
-    def test_madvise(self):
-        import mmap, sys
-        m = mmap.mmap(-1, 1024)
-        if not hasattr(m, "madvise"):
-            m.close()
-            skip("no madvise")
-
-        m.madvise(mmap.MADV_NORMAL)
-        m.close()
-
-    def test_repr(self):
-        import mmap
-        m = mmap.mmap(-1, 1024)
-        assert repr(m) == "<mmap.mmap closed=False, access=ACCESS_DEFAULT, length=1024, pos=0, offset=0>"
-        m.close()
-        assert repr(m) == "<mmap.mmap closed=True>"
