@@ -3,12 +3,22 @@ from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.typedef import TypeDef, interp_attrproperty
 from pypy.interpreter.error import OperationError, oefmt
-from rpython.rlib.rarithmetic import intmask, r_uint, r_uint32
+from rpython.rlib.rarithmetic import intmask, r_uint
 from rpython.rlib.objectmodel import keepalive_until_here
+from rpython.rtyper.lltypesystem import rffi
 
 from rpython.rlib import rzlib
 
 
+if intmask(2**31) == -2**31:
+    # 32-bit platforms
+    unsigned_to_signed_32bit = intmask
+else:
+    # 64-bit platforms
+    def unsigned_to_signed_32bit(x):
+        return intmask(rffi.cast(rffi.INT, x))
+
+
 @unwrap_spec(string='bufferstr', start='truncatedint_w')
 def crc32(space, string, start = rzlib.CRC32_DEFAULT_START):
     """
@@ -17,8 +27,16 @@ def crc32(space, string, start = rzlib.CRC32_DEFAULT_START):
     An optional starting value can be specified.  The returned checksum is
     an integer.
     """
-    ustart = r_uint(r_uint32(start))
+    ustart = r_uint(start)
     checksum = rzlib.crc32(string, ustart)
+
+    # This is, perhaps, a little stupid.  zlib returns the checksum unsigned.
+    # CPython exposes it as a signed value, though. -exarkun
+    # Note that in CPython < 2.6 on 64-bit platforms the result is
+    # actually unsigned, but it was considered to be a bug so we stick to
+    # the 2.6 behavior and always return a number in range(-2**31, 2**31).
+    checksum = unsigned_to_signed_32bit(checksum)
+
     return space.newint(checksum)
 
 
@@ -30,8 +48,11 @@ def adler32(space, string, start=rzlib.ADLER32_DEFAULT_START):
     An optional starting value can be specified.  The returned checksum is
     an integer.
     """
-    ustart = r_uint(r_uint32(start))
+    ustart = r_uint(start)
     checksum = rzlib.adler32(string, ustart)
+    # See comments in crc32() for the following line
+    checksum = unsigned_to_signed_32bit(checksum)
+
     return space.newint(checksum)
 
 
@@ -117,12 +138,11 @@ class Compress(ZLibObject):
                  method=rzlib.Z_DEFLATED,             # \
                  wbits=rzlib.MAX_WBITS,               #  \   undocumented
                  memLevel=rzlib.DEF_MEM_LEVEL,        #  /    parameters
-                 strategy=rzlib.Z_DEFAULT_STRATEGY,   # /
-                 zdict=None):
+                 strategy=rzlib.Z_DEFAULT_STRATEGY):  # /
         ZLibObject.__init__(self, space)
         try:
             self.stream = rzlib.deflateInit(level, method, wbits,
-                                            memLevel, strategy, zdict=zdict)
+                                            memLevel, strategy)
         except rzlib.RZlibError as e:
             raise zlib_error(space, e.msg)
         except ValueError:
@@ -194,19 +214,14 @@ def Compress___new__(space, w_subtype, level=rzlib.Z_DEFAULT_COMPRESSION,
                      method=rzlib.Z_DEFLATED,             # \
                      wbits=rzlib.MAX_WBITS,               #  \   undocumented
                      memLevel=rzlib.DEF_MEM_LEVEL,        #  /    parameters
-                     strategy=rzlib.Z_DEFAULT_STRATEGY,   # /
-                     w_zdict=None):
+                     strategy=rzlib.Z_DEFAULT_STRATEGY):  # /
     """
     Create a new z_stream and call its initializer.
     """
-    if space.is_none(w_zdict):
-        zdict = None
-    else:
-        zdict = space.charbuf_w(w_zdict)
     stream = space.allocate_instance(Compress, w_subtype)
     stream = space.interp_w(Compress, stream)
     Compress.__init__(stream, space, level,
-                      method, wbits, memLevel, strategy, zdict)
+                      method, wbits, memLevel, strategy)
     return stream
 
 
@@ -226,7 +241,7 @@ class Decompress(ZLibObject):
     Wrapper around zlib's z_stream structure which provides convenient
     decompression functionality.
     """
-    def __init__(self, space, wbits=rzlib.MAX_WBITS, zdict=None):
+    def __init__(self, space, wbits=rzlib.MAX_WBITS):
         """
         Initialize a new decompression object.
 
@@ -238,14 +253,12 @@ class Decompress(ZLibObject):
         ZLibObject.__init__(self, space)
         self.unused_data = ''
         self.unconsumed_tail = ''
-        self.eof = False
         try:
-            self.stream = rzlib.inflateInit(wbits, zdict=zdict)
+            self.stream = rzlib.inflateInit(wbits)
         except rzlib.RZlibError as e:
             raise zlib_error(space, e.msg)
         except ValueError:
             raise oefmt(space.w_ValueError, "Invalid initialization option")
-        self.zdict = zdict
         self.register_finalizer(space)
 
     def _finalize_(self):
@@ -282,16 +295,13 @@ class Decompress(ZLibObject):
         try:
             self.lock()
             try:
-                result = rzlib.decompress(self.stream, data,
-                                          max_length=max_length,
-                                          zdict=self.zdict)
+                result = rzlib.decompress(self.stream, data, max_length=max_length)
             finally:
                 self.unlock()
         except rzlib.RZlibError as e:
             raise zlib_error(space, e.msg)
 
         string, finished, unused_len = result
-        self.eof = finished
         self._save_unconsumed_input(data, finished, unused_len)
         return space.newbytes(string)
 
@@ -310,8 +320,7 @@ class Decompress(ZLibObject):
         try:
             self.lock()
             try:
-                result = rzlib.decompress(self.stream, data, rzlib.Z_FINISH,
-                                          zdict=self.zdict)
+                result = rzlib.decompress(self.stream, data, rzlib.Z_FINISH)
             finally:
                 self.unlock()
         except rzlib.RZlibError:
@@ -323,21 +332,15 @@ class Decompress(ZLibObject):
 
 
 @unwrap_spec(wbits=int)
-def Decompress___new__(space, w_subtype, wbits=rzlib.MAX_WBITS, w_zdict=None):
+def Decompress___new__(space, w_subtype, wbits=rzlib.MAX_WBITS):
     """
     Create a new Decompress and call its initializer.
     """
-    if space.is_none(w_zdict):
-        zdict = None
-    else:
-        zdict = space.charbuf_w(w_zdict)
     stream = space.allocate_instance(Decompress, w_subtype)
     stream = space.interp_w(Decompress, stream)
-    Decompress.__init__(stream, space, wbits, zdict)
+    Decompress.__init__(stream, space, wbits)
     return stream
 
-def default_buffer_size(space):
-    return space.newint(rzlib.OUTPUT_BUFFER_SIZE)
 
 Decompress.typedef = TypeDef(
     'Decompress',
@@ -346,7 +349,6 @@ Decompress.typedef = TypeDef(
     flush = interp2app(Decompress.flush),
     unused_data = interp_attrproperty('unused_data', Decompress, wrapfn="newbytes"),
     unconsumed_tail = interp_attrproperty('unconsumed_tail', Decompress, wrapfn="newbytes"),
-    eof = interp_attrproperty('eof', Decompress, wrapfn="newbool"),
     __doc__ = """decompressobj([wbits]) -- Return a decompressor object.
 
 Optional arg wbits is the window buffer size.
