@@ -1,16 +1,13 @@
 # encoding: utf-8
+import py
 import sys
 from pypy.interpreter.error import OperationError
 from pypy.objspace.std import intobject as iobj
 from rpython.rlib.rarithmetic import r_uint, is_valid_int, intmask
 from rpython.rlib.rbigint import rbigint
 
-from rpython.jit.metainterp.test.support import LLJitMixin, noConst
-
-from hypothesis import given, strategies
 
 class TestW_IntObject:
-
     def _longshiftresult(self, x):
         """ calculate an overflowing shift """
         n = 1
@@ -80,7 +77,7 @@ class TestW_IntObject:
         f1 = iobj.W_IntObject(x)
         f2 = iobj.W_IntObject(y)
         v = f1.descr_add(space, f2)
-        assert space.isinstance_w(v, space.w_int)
+        assert space.isinstance_w(v, space.w_long)
         assert space.bigint_w(v).eq(rbigint.fromlong(x + y))
 
     def test_sub(self):
@@ -96,7 +93,7 @@ class TestW_IntObject:
         f1 = iobj.W_IntObject(x)
         f2 = iobj.W_IntObject(y)
         v = f1.descr_sub(space, f2)
-        assert space.isinstance_w(v, space.w_int)
+        assert space.isinstance_w(v, space.w_long)
         assert space.bigint_w(v).eq(rbigint.fromlong(sys.maxint - -1))
 
     def test_mul(self):
@@ -112,9 +109,25 @@ class TestW_IntObject:
         f1 = iobj.W_IntObject(x)
         f2 = iobj.W_IntObject(y)
         v = f1.descr_mul(space, f2)
-        assert space.isinstance_w(v, space.w_int)
+        assert space.isinstance_w(v, space.w_long)
         assert space.bigint_w(v).eq(rbigint.fromlong(x * y))
 
+    def test_div(self):
+        space = self.space
+        for i in range(10):
+            res = i//3
+            f1 = iobj.W_IntObject(i)
+            f2 = iobj.W_IntObject(3)
+            result = f1.descr_div(space, f2)
+            assert result.intval == res
+        x = -sys.maxint-1
+        y = -1
+        f1 = iobj.W_IntObject(x)
+        f2 = iobj.W_IntObject(y)
+        v = f1.descr_div(space, f2)
+        assert space.isinstance_w(v, space.w_long)
+        assert space.bigint_w(v).eq(rbigint.fromlong(x / y))
+
     def test_mod(self):
         x = 1
         y = 2
@@ -139,7 +152,7 @@ class TestW_IntObject:
         f2 = iobj.W_IntObject(y)
         v = f1.descr_divmod(space, f2)
         w_q, w_r = space.fixedview(v, 2)
-        assert space.isinstance_w(w_q, space.w_int)
+        assert space.isinstance_w(w_q, space.w_long)
         expected = divmod(x, y)
         assert space.bigint_w(w_q).eq(rbigint.fromlong(expected[0]))
         # no overflow possible
@@ -155,7 +168,7 @@ class TestW_IntObject:
         v = f1.descr_pow(self.space, f2, f3)
         assert v.intval == pow(x, y, z)
         f1, f2, f3 = [iobj.W_IntObject(i) for i in (10, -1, 42)]
-        self.space.raises_w(self.space.w_ValueError,
+        self.space.raises_w(self.space.w_TypeError,
                             f1.descr_pow, self.space, f2, f3)
         f1, f2, f3 = [iobj.W_IntObject(i) for i in (10, 5, 0)]
         self.space.raises_w(self.space.w_ValueError,
@@ -171,7 +184,7 @@ class TestW_IntObject:
         assert v.intval == x ** y
         f1, f2 = [iobj.W_IntObject(i) for i in (10, 20)]
         v = f1.descr_pow(space, f2, space.w_None)
-        assert space.isinstance_w(v, space.w_int)
+        assert space.isinstance_w(v, space.w_long)
         assert space.bigint_w(v).eq(rbigint.fromlong(pow(10, 20)))
 
     try:
@@ -181,7 +194,7 @@ class TestW_IntObject:
     else:
         @given(
            a=strategies.integers(min_value=-sys.maxint-1, max_value=sys.maxint),
-           b=strategies.integers(min_value=0, max_value=sys.maxint),
+           b=strategies.integers(min_value=-sys.maxint-1, max_value=sys.maxint),
            c=strategies.integers(min_value=-sys.maxint-1, max_value=sys.maxint))
         @example(0, 0, -sys.maxint-1)
         @example(0, 1, -sys.maxint-1)
@@ -190,10 +203,10 @@ class TestW_IntObject:
             if c == 0:
                 return
             #
-            # "pow(a, b, c)": if b < 0, should get an app-level ValueError.
+            # "pow(a, b, c)": if b < 0, should get an app-level TypeError.
             # Otherwise, should always work except if c == -maxint-1
             if b < 0:
-                expected = "app-level ValueError"
+                expected = TypeError
             elif b > 0 and c == -sys.maxint-1:
                 expected = OverflowError
             else:
@@ -202,25 +215,13 @@ class TestW_IntObject:
             try:
                 result = iobj._pow(self.space, a, b, c)
             except OperationError as e:
-                assert ('ValueError: pow() 2nd argument cannot be negative '
+                assert ('TypeError: pow() 2nd argument cannot be negative '
                         'when 3rd argument specified' == e.errorstr(self.space))
-                result = "app-level ValueError"
+                result = TypeError
             except OverflowError:
                 result = OverflowError
             assert result == expected
 
-            # check exponent -1
-            if isinstance(expected, int):
-                try:
-                    result = iobj._pow(self.space, a, -1, c)
-                except OperationError as e:
-                    pass
-                except OverflowError:
-                    pass
-                else:
-                    assert (a * result % c) == 1 % c
-
-
         @given(
            a=strategies.integers(min_value=-sys.maxint-1, max_value=sys.maxint),
            b=strategies.integers(min_value=-sys.maxint-1, max_value=sys.maxint))
@@ -252,7 +253,7 @@ class TestW_IntObject:
         x = -sys.maxint-1
         f1 = iobj.W_IntObject(x)
         v = f1.descr_neg(space)
-        assert space.isinstance_w(v, space.w_int)
+        assert space.isinstance_w(v, space.w_long)
         assert space.bigint_w(v).eq(rbigint.fromlong(-x))
 
     def test_pos(self):
@@ -278,7 +279,7 @@ class TestW_IntObject:
         x = -sys.maxint-1
         f1 = iobj.W_IntObject(x)
         v = f1.descr_abs(space)
-        assert space.isinstance_w(v, space.w_int)
+        assert space.isinstance_w(v, space.w_long)
         assert space.bigint_w(v).eq(rbigint.fromlong(abs(x)))
 
     def test_invert(self):
@@ -299,7 +300,7 @@ class TestW_IntObject:
         f1 = iobj.W_IntObject(x)
         f2 = iobj.W_IntObject(y)
         v = f1.descr_lshift(space, f2)
-        assert space.isinstance_w(v, space.w_int)
+        assert space.isinstance_w(v, space.w_long)
         assert space.bigint_w(v).eq(rbigint.fromlong(x << y))
 
     def test_rshift(self):
@@ -339,6 +340,19 @@ class TestW_IntObject:
         result = f1.int(self.space)
         assert result == f1
 
+    def test_oct(self):
+        x = 012345
+        f1 = iobj.W_IntObject(x)
+        result = f1.descr_oct(self.space)
+        assert self.space.unwrap(result) == oct(x)
+
+    def test_hex(self):
+        x = 0x12345
+        f1 = iobj.W_IntObject(x)
+        result = f1.descr_hex(self.space)
+        assert self.space.unwrap(result) == hex(x)
+
+
 class AppTestInt(object):
     def test_hash(self):
         assert hash(-1) == (-1).__hash__() == -2
@@ -363,9 +377,6 @@ class AppTestInt(object):
         assert a == 2
         a -= 1
         assert a == 1
-        e = raises(TypeError, "a += 'aa'")
-        assert "+=" in str(e.value)
-
 
     def test_trunc(self):
         import math
@@ -383,19 +394,10 @@ class AppTestInt(object):
 
     def test_int_string(self):
         assert 42 == int("42")
-        assert 10000000000 == int("10000000000")
+        assert 10000000000 == long("10000000000")
 
-    def test_int_string_limit(self):
-        import sys
-        max_str_digits = sys.get_int_max_str_digits()
-        raises(ValueError, int, '1' * (max_str_digits + 1))
-        # should not fail
-        x = int(' ' + '1' * max_str_digits)
-        sys.set_int_max_str_digits(0)
-        try:
-            x = int('1' * (max_str_digits + 1))
-        finally:
-            sys.set_int_max_str_digits(max_str_digits)
+    def test_int_unicode(self):
+        assert 42 == int(unicode('42'))
 
     def test_int_float(self):
         assert 4 == int(4.2)
@@ -414,7 +416,7 @@ class AppTestInt(object):
 
     def test_int_largenums(self):
         import sys
-        for x in [-sys.maxsize-1, -1, sys.maxsize]:
+        for x in [-sys.maxint-1, -1, sys.maxint]:
             y = int(str(x))
             assert y == x
             assert type(y) is int
@@ -425,15 +427,15 @@ class AppTestInt(object):
 
     def test_overflow(self):
         import sys
-        n = sys.maxsize + 1
-        assert isinstance(n, int)
+        n = sys.maxint + 1
+        assert isinstance(n, long)
 
     def test_pow(self):
         assert pow(2, -10) == 1/1024.
 
     def test_int_w_long_arg(self):
-        assert int(10000000000) == 10000000000
-        assert int("10000000000") == 10000000000
+        assert int(10000000000) == 10000000000L
+        assert int("10000000000") == 10000000000l
         raises(ValueError, int, "10000000000JUNK")
         raises(ValueError, int, "10000000000JUNK", 10)
 
@@ -443,10 +445,12 @@ class AppTestInt(object):
             pass
         assert j(100) == 100
         assert isinstance(j(100),j)
-        assert j(100) == 100
+        assert j(100L) == 100
         assert j("100") == 100
         assert j("100",2) == 4
         assert isinstance(j("100",2),j)
+        raises(OverflowError,j,sys.maxint+1)
+        raises(OverflowError,j,str(sys.maxint+1))
 
     def test_int_subclass_ops(self):
         import sys
@@ -478,7 +482,7 @@ class AppTestInt(object):
         assert (5 +  j(100),  type(5 +  j(100))) == (     105, int)
         assert (5 -  j(100),  type(5 -  j(100))) == (     -95, int)
         assert (5 *  j(100),  type(5 *  j(100))) == (     500, int)
-        assert (5 << j(100),  type(5 << j(100))) == (5 << 100, int)
+        assert (5 << j(100),  type(5 << j(100))) == (5 << 100, long)
         assert (j(100) >> 2,  type(j(100) >> 2)) == (      25, int)
 
     def test_int_subclass_int(self):
@@ -489,15 +493,17 @@ class AppTestInt(object):
                 return '<instance of j>'
         class subint(int):
             pass
-        value = 42
+        class sublong(long):
+            pass
+        value = 42L
         assert int(j()) == 42
-        value = 4200000000000000000000000000000000
-        assert int(j()) == 4200000000000000000000000000000000
+        value = 4200000000000000000000000000000000L
+        assert int(j()) == 4200000000000000000000000000000000L
         value = subint(42)
-        assert int(j()) == 42 and type(int(j())) is int
-        value = subint(4200000000000000000000000000000000)
-        assert (int(j()) == 4200000000000000000000000000000000
-                and type(int(j())) is int)
+        assert int(j()) == 42 and type(int(j())) is subint
+        value = sublong(4200000000000000000000000000000000L)
+        assert (int(j()) == 4200000000000000000000000000000000L
+                and type(int(j())) is sublong)
         value = 42.0
         raises(TypeError, int, j())
         value = "foo"
@@ -518,16 +524,16 @@ class AppTestInt(object):
 
     def test_special_long(self):
         class a(object):
-            def __int__(self):
+            def __long__(self):
                 self.ar = True
                 return None
         inst = a()
-        raises(TypeError, int, inst)
+        raises(TypeError, long, inst)
         assert inst.ar == True
 
         class b(object):
             pass
-        raises((AttributeError,TypeError), int, b())
+        raises((AttributeError,TypeError), long, b())
 
     def test_just_trunc(self):
         class myint(object):
@@ -562,29 +568,7 @@ class AppTestInt(object):
                     return True
             n = int(TruncReturnsNonInt())
             assert n == 1
-            assert type(n) is int
-
-    def test_trunc_returns_int_subclass_2(self):
-        class BadInt:
-            def __int__(self):
-                return True
-
-        class TruncReturnsBadInt:
-            def __trunc__(self):
-                return BadInt()
-        bad_int = TruncReturnsBadInt()
-        n = int(bad_int)
-        assert n == 1
-        assert type(n) is int
-
-    def test_trunc_returns_index(self):
-        class Index:
-            def __index__(self):
-                return 17
-        class TruncReturnsIndex:
-            def __trunc__(self):
-                return Index()
-        assert int(TruncReturnsIndex()) == 17
+            assert type(n) is bool
 
     def test_int_before_string(self):
         class Integral(str):
@@ -595,6 +579,14 @@ class AppTestInt(object):
     def test_getnewargs(self):
         assert  0 .__getnewargs__() == (0,)
 
+    def test_cmp(self):
+        skip("This is a 'wont fix' case")
+        # We don't have __cmp__, we consistently have __eq__ & the others
+        # instead.  In CPython some types have __cmp__ and some types have
+        # __eq__ & the others.
+        assert 1 .__cmp__
+        assert int .__cmp__
+
     def test_bit_length(self):
         for val, bits in [
             (0, 0),
@@ -612,7 +604,7 @@ class AppTestInt(object):
 
     def test_bit_length_max(self):
         import sys
-        val = -sys.maxsize-1
+        val = -sys.maxint-1
         bits = 32 if val == -2147483648 else 64
         assert val.bit_length() == bits
 
@@ -623,14 +615,14 @@ class AppTestInt(object):
 
     def test_int_error_msg(self):
         e = raises(TypeError, int, [])
-        assert str(e.value) == ("int() argument must be a string, a bytes-"
-                                "like object or a number, not 'list'")
+        assert str(e.value) == (
+            "int() argument must be a string or a number, not 'list'")
 
     def test_invalid_literal_message(self):
         import sys
         if '__pypy__' not in sys.builtin_module_names:
             skip('PyPy 2.x/CPython 3.4 only')
-        for value in b'  1j ', '  1٢٣٤j ':
+        for value in b'  1j ', u'  1٢٣٤j ':
             try:
                 int(value)
             except ValueError as e:
@@ -638,75 +630,20 @@ class AppTestInt(object):
             else:
                 assert False, value
 
-    def test_int_error_msg_surrogate(self):
-        value = u'123\ud800'
-        e = raises(ValueError, int, value)
-        assert str(e.value) == u"invalid literal for int() with base 10: %r" % value
-        e = raises(ValueError, int, value, 10)
-        assert str(e.value) == u"invalid literal for int() with base 10: %r" % value
-
-    def test_non_numeric_input_types(self):
-        # Test possible non-numeric types for the argument x, including
-        # subclasses of the explicitly documented accepted types.
-        class CustomStr(str): pass
-        class CustomBytes(bytes): pass
-        class CustomByteArray(bytearray): pass
-
-        factories = [
-            bytes,
-            bytearray,
-            lambda b: CustomStr(b.decode()),
-            CustomBytes,
-            CustomByteArray,
-            memoryview,
-        ]
-        try:
-            from array import array
-        except ImportError:
-            pass
-        else:
-            factories.append(lambda b: array('B', b))
-
-        for f in factories:
-            x = f(b'100')
-            assert int(x) == 100
-            if isinstance(x, (str, bytes, bytearray)):
-                assert int(x, 2) == 4
-            else:
-                try:
-                    int(x, 2)
-                except TypeError as e:
-                    assert "can't convert non-string" in str(e)
-                else:
-                    assert False, 'did not raise'
-            try:
-                int(f(b'A' * 0x10))
-            except ValueError as e:
-                assert "invalid literal" in str(e)
-            else:
-                assert False, 'did not raise'
+    def test_coerce(self):
+        assert 3 .__coerce__(4) == (3, 4)
+        assert 3 .__coerce__(4L) == NotImplemented
 
     def test_fake_int_as_base(self):
         class MyInt(object):
             def __init__(self, x):
                 self.x = x
-            def __index__(self):
+            def __int__(self):
                 return self.x
 
         base = MyInt(24)
         assert int('10', base) == 24
 
-        class MyNonIndexable(object):
-            def __init__(self, x):
-                self.x = x
-            def __int__(self):
-                return self.x
-
-        base = MyNonIndexable(24)
-        e = raises(TypeError, int, '10', base)
-        assert str(e.value) == ("'MyNonIndexable' object cannot be interpreted "
-                                "as an integer")
-
     def test_truediv(self):
         import operator
         x = 1000000
@@ -736,219 +673,11 @@ class AppTestInt(object):
 
     def test_binop_overflow(self):
         x = int(2)
-        assert x.__lshift__(128) == 680564733841876926926749214863536422912
+        assert x.__lshift__(128) == 680564733841876926926749214863536422912L
 
     def test_rbinop_overflow(self):
         x = int(321)
-        assert x.__rlshift__(333) == 1422567365923326114875084456308921708325401211889530744784729710809598337369906606315292749899759616
-
-    def test_ceil(self):
-        assert 8 .__ceil__() == 8
-
-    def test_floor(self):
-        assert 8 .__floor__() == 8
-
-    def test_deprecation_warning_1(self):
-        import warnings, _operator
-        class BadInt:
-            def __int__(self):
-                return True
-            def __index__(self):
-                return False
-        bad = BadInt()
-        with warnings.catch_warnings(record=True) as log:
-            warnings.simplefilter("always", DeprecationWarning)
-            n = int(bad)
-            m = _operator.index(bad)
-        assert n == 1 and type(n) is int
-        assert m is False
-        assert len(log) == 2
-
-    def test_deprecation_warning_2(self):
-        import warnings, _operator
-        class BadInt(int):
-            def __int__(self):
-                return self
-            def __index__(self):
-                return self
-        bad = BadInt(1)
-        with warnings.catch_warnings(record=True) as log:
-            warnings.simplefilter("always", DeprecationWarning)
-            n = int(bad)
-            m = _operator.index(bad)  # no warning
-        assert n == 1 and type(n) is int
-        assert m is bad
-        assert len(log) == 1
-        assert log[0].message.args[0].startswith('__int__')
-
-    def test_deprecation_warning_3(self):
-        import warnings, _operator
-        class BadInt(int):
-            def __int__(self):
-                return self
-            def __index__(self):
-                return self
-        bad = BadInt(2**100)
-        with warnings.catch_warnings(record=True) as log:
-            warnings.simplefilter("always", DeprecationWarning)
-            n = int(bad)
-            m = _operator.index(bad)  # no warning
-        assert n == bad and type(n) is int
-        assert m is bad
-        assert len(log) == 1
-        assert log[0].message.args[0].startswith('__int__')
-
-    def test_int_nonstr_with_base(self):
-        assert int(b'100', 2) == 4
-        assert int(bytearray(b'100'), 2) == 4
-        raises(TypeError, int, memoryview(b'100'), 2)
-
-    def test_from_bytes(self):
-        called = []
-        class X(int):
-            def __init__(self, val):
-                called.append(val)
-        x = X.from_bytes(b"", 'little')
-        assert type(x) is X and x == 0
-        assert called == [0]
-        x = X.from_bytes(b"*" * 100, 'little')
-        assert type(x) is X
-        expected = sum(256 ** i for i in range(100)) * ord('*')
-        assert x == expected
-        assert called == [0, expected]
-
-    def test_leading_zero_literal(self):
-        assert eval("00") == 0
-        raises(SyntaxError, eval, '07')
-        assert int("00", 0) == 0
-        raises(ValueError, int, '07', 0)
-        assert int("07", 10) == 7
-        raises(ValueError, int, '07777777777777777777777777777777777777', 0)
-        raises(ValueError, int, '00000000000000000000000000000000000007', 0)
-        raises(ValueError, int, '00000000000000000077777777777777777777', 0)
-
-    def test_some_rops(self):
-        b = 2 ** 31
-        x = -b
-        assert x.__rsub__(2) == (2 + b)
-
-    def test_round_special_method(self):
-        assert 567 .__round__(-1) == 570
-        assert 567 .__round__() == 567
-        import sys
-        if '__pypy__' in sys.builtin_module_names:
-            assert 567 .__round__(None) == 567    # fails on CPython
-
-
-    def test_error_message_wrong_self(self):
-        unboundmeth = int.__str__
-        e = raises(TypeError, unboundmeth, "!")
-        assert "int" in str(e.value)
-        if hasattr(unboundmeth, 'im_func'):
-            e = raises(TypeError, unboundmeth.im_func, "!")
-            assert "'int'" in str(e.value)
-
-    def test_int_new_pos_only(self):
-        with raises(TypeError) as info:
-            int(x=1)
-        assert "got a positional-only argument passed as keyword argument: 'x'" in str(info.value)
-
-    def test_int_as_integer_ratio(self):
-        assert 4 .as_integer_ratio() == (4, 1)
-        assert (-1).as_integer_ratio() == (-1, 1)
-        assert (2 ** 100).as_integer_ratio() == (2 ** 100).as_integer_ratio()
-
-        d, n = True.as_integer_ratio()
-        assert (d, n) == (1, 1)
-        assert type(d) is int
-        d, n = False.as_integer_ratio()
-        assert (d, n) == (0, 1)
-        assert type(d) is int
-
-        class X(int): pass
-        a = X(5)
-        n, d = a.as_integer_ratio()
-        assert n == 5 and d == 1
-        assert type(n) is int
-
-    def test_pow_negative_exponent_mod(self):
-        import math
-        for i in range(1, 7):
-            for j in range(2, 7):
-                for sign in [1, -1]:
-                    i = i * sign
-                    if math.gcd(i, j) != 1:
-                        with raises(ValueError):
-                            pow(i, -1, j)
-                        continue
-                    x = pow(i, -1, j)
-                    assert (x * i) % j == 1 % j
-
-                    for k in range(2, 4):
-                        y = pow(i, -k, j)
-                        assert y == pow(x, k, j)
-
-    def test_int_constructor_calls_index(self):
-        class A:
-            def __index__(self):
-                return 25
-        assert int(A()) == 25
-        reallybig = 1 << 1000
-        class A:
-            def __index__(self):
-                return reallybig
-        assert int(A()) == reallybig
-
-        class A:
-            def __index__(self):
-                return "abc"
-        with raises(TypeError):
-            int(A())
-
-        class subint(int):
-            pass
-        class A:
-            def __index__(self):
-                return subint(12)
-        x = int(A())
-        assert x == 12
-        assert type(x) is int
-
-    def test_bit_count(self):
-        for x in (42, 2**100, 2**63, 2**63-1, 2**31-1, 2**31):
-            assert x.bit_count() == bin(x).count("1")
-            assert (-x).bit_count() == bin(x).count("1")
-
-
-def test_hash_examples():
-    for i in range(1000):
-        assert iobj._hash_int(i) == i
-        assert iobj._hash_int(-i-2) == -i-2
-    assert iobj._hash_int(-1) == -2
-
-    # now bigger/smaller than HASH_MODULUS
-    for i in range(1000):
-        input = iobj.HASH_MODULUS + i
-        assert iobj._hash_int(input) == i
-
-        input = -iobj.HASH_MODULUS-i-1
-        if -i-1 == -1:
-            assert iobj._hash_int(input) == -2
-        else:
-            assert iobj._hash_int(input) == -i-1
-    assert iobj._hash_int(-sys.maxsize-1) == -4
-
-@given(strategies.integers(min_value=-sys.maxsize-1, max_value=sys.maxsize))
-def test_agreement_rbigint_hash(x):
-    from pypy.objspace.std.longobject import _hash_long
-    assert iobj._hash_int(x) == _hash_long(rbigint.fromint(x))
-    
-class TestHashNoBranch(LLJitMixin):
-    def test_int_hash_no_guards(self):
-        res = self.interp_operations(iobj._hash_int, [123])
-        assert res == 123
-        self.check_operations_history(guard_true=0, guard_false=0)
-
+        assert x.__rlshift__(333) == 1422567365923326114875084456308921708325401211889530744784729710809598337369906606315292749899759616L
 
 class AppTestIntShortcut(AppTestInt):
     spaceconfig = {"objspace.std.intshortcut": True}
@@ -956,7 +685,7 @@ class AppTestIntShortcut(AppTestInt):
     def test_inplace(self):
         # ensure other inplace ops still work
         l = []
-        l += range(5)
+        l += xrange(5)
         assert l == list(range(5))
         a = 8.5
         a -= .5
