@@ -9,36 +9,31 @@ from pypy.interpreter.gateway import (
 from pypy.interpreter.typedef import TypeDef
 from pypy.objspace.std.sliceobject import (W_SliceObject, unwrap_start_stop,
     normalize_simple_slice)
-from pypy.objspace.std.util import negate, IDTAG_SPECIAL, IDTAG_SHIFT, \
-    generic_alias_class_getitem
+from pypy.objspace.std.util import negate, IDTAG_SPECIAL, IDTAG_SHIFT
 from rpython.rlib import jit
 from rpython.rlib.debug import make_sure_not_resized
-from rpython.rlib.rarithmetic import intmask, r_ulonglong, r_uint
+from rpython.rlib.rarithmetic import intmask
 
 
 UNROLL_CUTOFF = 10
 
 
+def _unroll_condition(self):
+    return jit.loop_unrolling_heuristic(self, self.length(), UNROLL_CUTOFF)
+
+
 def _unroll_condition_cmp(self, space, other):
-    return self._unroll_condition() or other._unroll_condition()
+    return (jit.loop_unrolling_heuristic(self, self.length(), UNROLL_CUTOFF) or
+            jit.loop_unrolling_heuristic(other, other.length(), UNROLL_CUTOFF))
 
 
-def get_printable_location(tp):
-    return "tuple.contains [%s]" % (tp.getname(tp.space), )
-
-contains_driver = jit.JitDriver(greens = ['tp'], reds = 'auto',
-                             name = 'tuple.contains',
-                             get_printable_location=get_printable_location)
-
-def get_printable_location(w_type):
-    return "tuple.hash [%s]" % (w_type.getname(w_type.space), )
+contains_jmp = jit.JitDriver(greens = ['tp'], reds = 'auto',
+                             name = 'tuple.contains')
 
 hash_driver = jit.JitDriver(
     name='tuple.hash',
     greens=['w_type'],
-    reds='auto',
-    get_printable_location=get_printable_location
-    )
+    reds='auto')
 
 class W_AbstractTupleObject(W_Root):
     __slots__ = ()
@@ -92,7 +87,7 @@ class W_AbstractTupleObject(W_Root):
         return iterobject.W_FastTupleIterObject(self, self.tolist())
 
     @staticmethod
-    def descr_new(space, w_tupletype, w_sequence=None, __posonly__=None):
+    def descr_new(space, w_tupletype, w_sequence=None):
         if w_sequence is None:
             tuple_w = []
         elif (space.is_w(w_tupletype, space.w_tuple) and
@@ -107,11 +102,9 @@ class W_AbstractTupleObject(W_Root):
     def descr_repr(self, space):
         items = self.tolist()
         if len(items) == 1:
-            return space.newtext(
-                b"(" + space.utf8_w(space.repr(items[0])) + b",)")
-        tmp = b", ".join([space.utf8_w(space.repr(item))
-                          for item in items])
-        return space.newtext(b"(" + tmp + b")")
+            return space.newtext("(" + space.text_w(space.repr(items[0])) + ",)")
+        tmp = ", ".join([space.text_w(space.repr(item)) for item in items])
+        return space.newtext("(" + tmp + ")")
 
     def descr_hash(self, space):
         raise NotImplementedError
@@ -152,7 +145,7 @@ class W_AbstractTupleObject(W_Root):
     descr_ge = _make_tuple_comparison('ge')
 
     def descr_contains(self, space, w_obj):
-        if self._unroll_condition():
+        if _unroll_condition(self):
             return self._descr_contains_unroll_safe(space, w_obj)
         else:
             return self._descr_contains_jmp(space, w_obj)
@@ -166,14 +159,10 @@ class W_AbstractTupleObject(W_Root):
 
     def _descr_contains_jmp(self, space, w_obj):
         tp = space.type(w_obj)
-        list_w = self.tolist()
-        i = 0
-        while i < len(list_w):
-            contains_driver.jit_merge_point(tp=tp)
-            w_item = list_w[i]
+        for w_item in self.tolist():
+            contains_jmp.jit_merge_point(tp=tp)
             if space.eq_w(w_obj, w_item):
                 return space.w_True
-            i += 1
         return space.w_False
 
     def descr_add(self, space, w_other):
@@ -198,35 +187,29 @@ class W_AbstractTupleObject(W_Root):
     def descr_getitem(self, space, w_index):
         if isinstance(w_index, W_SliceObject):
             return self._getslice(space, w_index)
-        index = space.getindex_w(w_index, space.w_IndexError, "tuple")
+        index = space.getindex_w(w_index, space.w_IndexError, "tuple index")
         return self.getitem(space, index)
 
     def _getslice(self, space, w_index):
         items = self.tolist()
         length = len(items)
         start, stop, step, slicelength = w_index.indices4(space, length)
-        if slicelength == 0:
-            subitems = []
-        elif step == 1:
-            assert 0 <= start <= stop
-            subitems = items[start:stop]
-        else:
-            subitems = self._getslice_advanced(items, start, step, slicelength)
-        return space.newtuple(subitems)
-
-    @staticmethod
-    def _getslice_advanced(items, start, step, slicelength):
         assert slicelength >= 0
         subitems = [None] * slicelength
         for i in range(slicelength):
             subitems[i] = items[start]
             start += step
-        return subitems
+        return space.newtuple(subitems)
+
+    def descr_getslice(self, space, w_start, w_stop):
+        length = self.length()
+        start, stop = normalize_simple_slice(space, length, w_start, w_stop)
+        return space.newtuple(self.tolist()[start:stop])
 
     def descr_getnewargs(self, space):
         return space.newtuple([space.newtuple(self.tolist())])
 
-    @jit.look_inside_iff(lambda self, _1, _2: self._unroll_condition())
+    @jit.look_inside_iff(lambda self, _1, _2: _unroll_condition(self))
     def descr_count(self, space, w_obj):
         """count(obj) -> number of times obj appears in the tuple"""
         count = 0
@@ -236,7 +219,7 @@ class W_AbstractTupleObject(W_Root):
         return space.newint(count)
 
     @unwrap_spec(w_start=WrappedDefault(0), w_stop=WrappedDefault(sys.maxint))
-    @jit.look_inside_iff(lambda self, _1, _2, _3, _4: self._unroll_condition())
+    @jit.look_inside_iff(lambda self, _1, _2, _3, _4: _unroll_condition(self))
     def descr_index(self, space, w_obj, w_start, w_stop):
         """index(obj, [start, [stop]]) -> first index that obj appears in the
         tuple
@@ -249,10 +232,6 @@ class W_AbstractTupleObject(W_Root):
                 return space.newint(i)
         raise oefmt(space.w_ValueError, "tuple.index(x): x not in tuple")
 
-    def _unroll_condition(self):
-        raise NotImplementedError("abstract base class")
-
-
 W_AbstractTupleObject.typedef = TypeDef(
     "tuple",
     __doc__ = """tuple() -> an empty tuple
@@ -279,28 +258,14 @@ If the argument is a tuple, the return value is the same object.""",
     __rmul__ = interp2app(W_AbstractTupleObject.descr_mul),
 
     __getitem__ = interp2app(W_AbstractTupleObject.descr_getitem),
+    __getslice__ = interp2app(W_AbstractTupleObject.descr_getslice),
 
     __getnewargs__ = interp2app(W_AbstractTupleObject.descr_getnewargs),
-    __class_getitem__ = interp2app(
-        generic_alias_class_getitem, as_classmethod=True),
-
     count = interp2app(W_AbstractTupleObject.descr_count),
     index = interp2app(W_AbstractTupleObject.descr_index)
 )
 W_AbstractTupleObject.typedef.flag_sequence_bug_compat = True
 
-if sys.maxint == 2 ** 31 - 1:
-    XXPRIME_1 = r_uint(2654435761)
-    XXPRIME_2 = r_uint(2246822519)
-    XXPRIME_5 = r_uint(374761393)
-    xxrotate = lambda x: ((x << 13) | (x >> 19)) # rotate left 13 bits
-    uhash_type = r_uint
-else:
-    XXPRIME_1 = r_ulonglong(11400714785074694791)
-    XXPRIME_2 = r_ulonglong(14029467366897019727)
-    XXPRIME_5 = r_ulonglong(2870177450012600261)
-    xxrotate = lambda x: ((x << 31) | (x >> 33)) # Rotate left 31 bits
-    uhash_type = r_ulonglong
 
 class W_TupleObject(W_AbstractTupleObject):
     _immutable_fields_ = ['wrappeditems[*]']
@@ -319,55 +284,37 @@ class W_TupleObject(W_AbstractTupleObject):
         return len(self.wrappeditems)
 
     def descr_hash(self, space):
-        if self._unroll_condition():
-            acc = self._descr_hash_unroll(space)
+        if _unroll_condition(self):
+            return self._descr_hash_unroll(space)
         else:
-            acc = self._descr_hash_jitdriver(space)
-
-        # Add input length, mangled to keep the historical value of hash(())
-        acc += len(self.wrappeditems) ^ (XXPRIME_5 ^ uhash_type(3527539))
-        acc += (acc == uhash_type(-1)) * uhash_type(1546275796 + 1)
-        return space.newint(intmask(acc))
+            return self._descr_hash_jitdriver(space)
 
     @jit.unroll_safe
     def _descr_hash_unroll(self, space):
-        # Hash for tuples. This is a slightly simplified version of the xxHash
-        # non-cryptographic hash:
-        # - we do not use any parallellism, there is only 1 accumulator.
-        # - we drop the final mixing since this is just a permutation of the
-        #   output space: it does not help against collisions.
-        # - at the end, we mangle the length with a single constant.
-        # For the xxHash specification, see
-        # https://github.com/Cyan4973/xxHash/blob/master/doc/xxhash_spec.md
-
-        # Below are the official constants from the xxHash specification. Optimizing
-        # compilers should emit a single "rotate" instruction for the
-        # _PyHASH_XXROTATE() expansion. If that doesn't happen for some important
-        # platform, the macro could be changed to expand to a platform-specific rotate
-        # spelling instead.
-        acc = XXPRIME_5
+        mult = 1000003
+        x = 0x345678
+        z = len(self.wrappeditems)
         for w_item in self.wrappeditems:
-            lane = uhash_type(space.hash_w(w_item))
-            acc += lane * XXPRIME_2
-            acc = xxrotate(acc)
-            acc *= XXPRIME_1
-        return acc
+            y = space.hash_w(w_item)
+            x = (x ^ y) * mult
+            z -= 1
+            mult += 82520 + z + z
+        x += 97531
+        return space.newint(intmask(x))
 
     def _descr_hash_jitdriver(self, space):
-        # see comments above
-        acc = XXPRIME_5
+        mult = 1000003
+        x = 0x345678
+        z = len(self.wrappeditems)
         w_type = space.type(self.wrappeditems[0])
-        wrappeditems = self.wrappeditems
-        i = 0
-        while i < len(wrappeditems):
+        for w_item in self.wrappeditems:
             hash_driver.jit_merge_point(w_type=w_type)
-            w_item = wrappeditems[i]
-            lane = uhash_type(space.hash_w(w_item))
-            acc += lane * XXPRIME_2
-            acc = xxrotate(acc)
-            acc *= XXPRIME_1
-            i += 1
-        return acc
+            y = space.hash_w(w_item)
+            x = (x ^ y) * mult
+            z -= 1
+            mult += 82520 + z + z
+        x += 97531
+        return space.newint(intmask(x))
 
     def descr_eq(self, space, w_other):
         if not isinstance(w_other, W_AbstractTupleObject):
@@ -382,7 +329,6 @@ class W_TupleObject(W_AbstractTupleObject):
         lgt2 = len(items2)
         if lgt1 != lgt2:
             return space.w_False
-        # XXX do we need a jit driver?
         for i in range(lgt1):
             item1 = items1[i]
             item2 = items2[i]
@@ -398,10 +344,6 @@ class W_TupleObject(W_AbstractTupleObject):
         except IndexError:
             raise oefmt(space.w_IndexError, "tuple index out of range")
 
-    def _unroll_condition(self):
-        return jit.loop_unrolling_heuristic(
-                self.wrappeditems, self.length(), UNROLL_CUTOFF)
-
 
 def wraptuple(space, list_w):
     if space.config.objspace.std.withspecialisedtuple:
@@ -411,12 +353,3 @@ def wraptuple(space, list_w):
         except NotSpecialised:
             pass
     return W_TupleObject(list_w)
-
-def wraptuple2(space, w_a, w_b):
-    if space.config.objspace.std.withspecialisedtuple:
-        from specialisedtupleobject import makespecialisedtuple2, NotSpecialised
-        try:
-            return makespecialisedtuple2(space, w_a, w_b)
-        except NotSpecialised:
-            pass
-    return W_TupleObject([w_a, w_b])
