@@ -11,7 +11,7 @@ from pypy.module.__builtin__ import abstractinst
 from rpython.rlib.jit import (promote, elidable_promote, we_are_jitted,
      elidable, dont_look_inside, unroll_safe)
 from rpython.rlib.objectmodel import current_object_addr_as_int, compute_hash
-from rpython.rlib.objectmodel import we_are_translated
+from rpython.rlib.objectmodel import we_are_translated, not_rpython
 from rpython.rlib.rarithmetic import intmask, r_uint
 
 class MutableCell(W_Root):
@@ -34,7 +34,7 @@ class IntMutableCell(MutableCell):
         self.intvalue = intvalue
 
     def unwrap_cell(self, space):
-        return space.wrap(self.intvalue)
+        return space.newint(self.intvalue)
 
     def __repr__(self):
         return "<IntMutableCell: %s>" % (self.intvalue, )
@@ -71,6 +71,10 @@ class VersionTag(object):
 class MethodCache(object):
 
     def __init__(self, space):
+        # Note: these attributes never change which object they contain,
+        # so reading 'cache.versions' for example is constant-folded.
+        # The actual list in 'cache.versions' is not a constant, of
+        # course.
         SIZE = 1 << space.config.objspace.std.methodcachesizeexp
         self.versions = [None] * SIZE
         self.names = [None] * SIZE
@@ -88,6 +92,9 @@ class MethodCache(object):
         for i in range(len(self.lookup_where)):
             self.lookup_where[i] = None_None
 
+    def _cleanup_(self):
+        self.clear()
+
 class _Global(object):
     weakref_warning_printed = False
 _global = _Global()
@@ -162,7 +169,8 @@ class W_TypeObject(W_Root):
 
     @dont_look_inside
     def __init__(self, space, name, bases_w, dict_w,
-                 overridetypedef=None, force_new_layout=False):
+                 overridetypedef=None, force_new_layout=False,
+                 is_heaptype=True):
         self.space = space
         self.name = name
         self.bases_w = bases_w
@@ -172,7 +180,7 @@ class W_TypeObject(W_Root):
         self.weakrefable = False
         self.w_doc = space.w_None
         self.weak_subclasses = []
-        self.flag_heaptype = False
+        self.flag_heaptype = is_heaptype
         self.flag_cpytype = False
         self.flag_abstract = False
         self.flag_sequence_bug_compat = False
@@ -204,8 +212,8 @@ class W_TypeObject(W_Root):
         else:
             self.terminator = NoDictTerminator(space, self)
 
+    @not_rpython
     def __repr__(self):
-        "NOT_RPYTHON"
         return '<W_TypeObject %r at 0x%x>' % (self.name, id(self))
 
     def mutated(self, key):
@@ -256,7 +264,8 @@ class W_TypeObject(W_Root):
                 # if it was not actually overriden in the class, we remember this
                 # fact for the next time.
                 if w_descr is object_getattribute(self.space):
-                    self.uses_object_getattribute = True
+                    if self.space._side_effects_ok():
+                        self.uses_object_getattribute = True
                 else:
                     return w_descr
             return None
@@ -283,10 +292,12 @@ class W_TypeObject(W_Root):
                                     self.lookup('__cmp__') or
                                     self.lookup('__hash__') is not default_hash)
         if overrides_eq_cmp_or_hash:
-            self.compares_by_identity_status = OVERRIDES_EQ_CMP_OR_HASH
+            result = OVERRIDES_EQ_CMP_OR_HASH
         else:
-            self.compares_by_identity_status = COMPARES_BY_IDENTITY
-        return self.compares_by_identity_status == COMPARES_BY_IDENTITY
+            result = COMPARES_BY_IDENTITY
+        if self.space._side_effects_ok():
+            self.compares_by_identity_status = result
+        return result == COMPARES_BY_IDENTITY
 
     def ready(self):
         for w_base in self.bases_w:
@@ -337,7 +348,7 @@ class W_TypeObject(W_Root):
         if name == "__del__" and name not in self.dict_w:
             msg = ("a __del__ method added to an existing type will not be "
                    "called")
-            space.warn(space.wrap(msg), space.w_RuntimeWarning)
+            space.warn(space.newtext(msg), space.w_RuntimeWarning)
         version_tag = self.version_tag()
         if version_tag is not None:
             w_curr = self._pure_getdictvalue_no_unwrapping(
@@ -461,11 +472,12 @@ class W_TypeObject(W_Root):
 #                print "hit", self, name
                 return tup
         tup = self._lookup_where_all_typeobjects(name)
-        cache.versions[method_hash] = version_tag
-        cache.names[method_hash] = name
-        cache.lookup_where[method_hash] = tup
-        if space.config.objspace.std.withmethodcachecounter:
-            cache.misses[name] = cache.misses.get(name, 0) + 1
+        if space._side_effects_ok():
+            cache.versions[method_hash] = version_tag
+            cache.names[method_hash] = name
+            cache.lookup_where[method_hash] = tup
+            if space.config.objspace.std.withmethodcachecounter:
+                cache.misses[name] = cache.misses.get(name, 0) + 1
 #        print "miss", self, name
         return tup
 
@@ -484,8 +496,9 @@ class W_TypeObject(W_Root):
                         self, w_subtype, w_subtype)
         return w_subtype
 
+    @not_rpython
     def _cleanup_(self):
-        "NOT_RPYTHON.  Forces the lazy attributes to be computed."
+        "Forces the lazy attributes to be computed."
         if 'lazyloaders' in self.__dict__:
             for attr in self.lazyloaders.keys():
                 self.getdictvalue(self.space, attr)
@@ -527,23 +540,27 @@ class W_TypeObject(W_Root):
         space = self.space
         if self.is_heaptype():
             return self.getdictvalue(space, '__module__')
+        elif self.is_cpytype():
+            dot = self.name.rfind('.')
         else:
             dot = self.name.find('.')
-            if dot >= 0:
-                mod = self.name[:dot]
-            else:
-                mod = "__builtin__"
-            return space.wrap(mod)
+        if dot >= 0:
+            mod = self.name[:dot]
+        else:
+            mod = "__builtin__"
+        return space.newtext(mod)
 
     def getname(self, space):
         if self.is_heaptype():
             return self.name
+        elif self.is_cpytype():
+            dot = self.name.rfind('.')
         else:
             dot = self.name.find('.')
-            if dot >= 0:
-                return self.name[dot+1:]
-            else:
-                return self.name
+        if dot >= 0:
+            return self.name[dot+1:]
+        else:
+            return self.name
 
     def add_subclass(self, w_subclass):
         space = self.space
@@ -613,9 +630,15 @@ class W_TypeObject(W_Root):
             if w_newdescr is None:    # see test_crash_mro_without_object_1
                 raise oefmt(space.w_TypeError, "cannot create '%N' instances",
                             self)
+            #
+            # issue #2666
+            if space.config.objspace.usemodules.cpyext:
+                w_newtype, w_newdescr = self.hack_which_new_to_call(
+                    w_newtype, w_newdescr)
+            #
             w_newfunc = space.get(w_newdescr, self)
             if (space.config.objspace.std.newshortcut and
-                not we_are_jitted() and
+                not we_are_jitted() and space._side_effects_ok() and
                 isinstance(w_newtype, W_TypeObject)):
                 self.w_new_function = w_newfunc
         w_newobject = space.call_obj_args(w_newfunc, self, __args__)
@@ -633,23 +656,63 @@ class W_TypeObject(W_Root):
                                 "__init__() should return None")
         return w_newobject
 
+    def hack_which_new_to_call(self, w_newtype, w_newdescr):
+        # issue #2666: for cpyext, we need to hack in order to reproduce
+        # an "optimization" of CPython that actually changes behaviour
+        # in corner cases.
+        #
+        # * Normally, we use the __new__ found in the MRO in the normal way.
+        #
+        # * If by chance this __new__ happens to be implemented as a C
+        #   function, then instead, we discard it and use directly
+        #   self.__base__.tp_new.
+        #
+        # * Most of the time this is the same (and faster for CPython), but
+        #   it can fail if self.__base__ happens not to be the first base.
+        #
+        from pypy.module.cpyext.methodobject import W_PyCFunctionObject
+
+        if isinstance(w_newdescr, W_PyCFunctionObject):
+            return self._really_hack_which_new_to_call(w_newtype, w_newdescr)
+        else:
+            return w_newtype, w_newdescr
+
+    def _really_hack_which_new_to_call(self, w_newtype, w_newdescr):
+        # This logic is moved in yet another helper function that
+        # is recursive.  We call this only if we see a
+        # W_PyCFunctionObject.  That's a performance optimization
+        # because in the common case, we won't call any function that
+        # contains the stack checks.
+        from pypy.module.cpyext.methodobject import W_PyCFunctionObject
+        from pypy.module.cpyext.typeobject import is_tp_new_wrapper
+
+        if (isinstance(w_newdescr, W_PyCFunctionObject) and
+                w_newtype is not self and
+                is_tp_new_wrapper(self.space, w_newdescr.ml)):
+            w_bestbase = find_best_base(self.bases_w)
+            if w_bestbase is not None:
+                w_newtype, w_newdescr = w_bestbase.lookup_where('__new__')
+                return w_bestbase._really_hack_which_new_to_call(w_newtype,
+                                                                 w_newdescr)
+        return w_newtype, w_newdescr
+
     def descr_repr(self, space):
         w_mod = self.get_module()
-        if w_mod is None or not space.isinstance_w(w_mod, space.w_str):
+        if w_mod is None or not space.isinstance_w(w_mod, space.w_text):
             mod = None
         else:
-            mod = space.str_w(w_mod)
+            mod = space.text_w(w_mod)
         if not self.is_heaptype():
             kind = 'type'
         else:
             kind = 'class'
         if mod is not None and mod != '__builtin__':
-            return space.wrap("<%s '%s.%s'>" % (kind, mod, self.getname(space)))
+            return space.newtext("<%s '%s.%s'>" % (kind, mod, self.getname(space)))
         else:
-            return space.wrap("<%s '%s'>" % (kind, self.name))
+            return space.newtext("<%s '%s'>" % (kind, self.name))
 
     def descr_getattribute(self, space, w_name):
-        name = space.str_w(w_name)
+        name = space.text_w(w_name)
         w_descr = space.lookup(self, name)
         if w_descr is not None:
             if space.is_data_descr(w_descr):
@@ -691,7 +754,7 @@ def descr__new__(space, w_typetype, w_name, w_bases=None, w_dict=None):
 def _check_new_args(space, w_name, w_bases, w_dict):
     if w_bases is None or w_dict is None:
         raise oefmt(space.w_TypeError, "type() takes 1 or 3 arguments")
-    if not space.isinstance_w(w_name, space.w_str):
+    if not space.isinstance_w(w_name, space.w_text):
         raise oefmt(space.w_TypeError,
                     "type() argument 1 must be string, not %T", w_name)
     if not space.isinstance_w(w_bases, space.w_tuple):
@@ -724,23 +787,23 @@ def _create_new_type(space, w_typetype, w_name, w_bases, w_dict):
                     " bases")
 
     if not space.is_w(w_winner, w_typetype):
-        newfunc = space.getattr(w_winner, space.wrap('__new__'))
-        if not space.is_w(newfunc, space.getattr(space.w_type, space.wrap('__new__'))):
+        newfunc = space.getattr(w_winner, space.newtext('__new__'))
+        if not space.is_w(newfunc, space.getattr(space.w_type, space.newtext('__new__'))):
             return space.call_function(newfunc, w_winner, w_name, w_bases, w_dict)
         w_typetype = w_winner
 
-    name = space.str_w(w_name)
+    name = space.text_w(w_name)
     assert isinstance(name, str)
     if '\x00' in name:
         raise oefmt(space.w_ValueError, "type name must not contain null characters")
     dict_w = {}
     dictkeys_w = space.listview(w_dict)
     for w_key in dictkeys_w:
-        key = space.str_w(w_key)
+        key = space.text_w(w_key)
         dict_w[key] = space.getitem(w_dict, w_key)
     w_type = space.allocate_instance(W_TypeObject, w_typetype)
     W_TypeObject.__init__(w_type, space, name, bases_w or [space.w_object],
-                          dict_w)
+                          dict_w, is_heaptype=True)
     w_type.ready()
     return w_type
 
@@ -763,23 +826,23 @@ def descr__init__(space, w_type, __args__):
 
 def _check(space, w_type, msg="descriptor is for 'type'"):
     if not isinstance(w_type, W_TypeObject):
-        raise OperationError(space.w_TypeError, space.wrap(msg))
+        raise OperationError(space.w_TypeError, space.newtext(msg))
     return w_type
 
 
 def descr_get__name__(space, w_type):
     w_type = _check(space, w_type)
-    return space.wrap(w_type.getname(space))
+    return space.newtext(w_type.getname(space))
 
 def descr_set__name__(space, w_type, w_value):
     w_type = _check(space, w_type)
     if not w_type.is_heaptype():
         raise oefmt(space.w_TypeError, "can't set %N.__name__", w_type)
-    if not space.isinstance_w(w_value, space.w_str):
+    if not space.isinstance_w(w_value, space.w_text):
         raise oefmt(space.w_TypeError,
                     "can only assign string to %N.__name__, not '%T'",
                     w_type, w_value)
-    name = space.str_w(w_value)
+    name = space.text_w(w_value)
     if '\x00' in name:
         raise oefmt(space.w_ValueError, "type name must not contain null characters")
     w_type.name = name
@@ -870,7 +933,7 @@ def descr__base(space, w_type):
 
 def descr__doc(space, w_type):
     if space.is_w(w_type, space.w_type):
-        return space.wrap("""type(object) -> the object's type
+        return space.newtext("""type(object) -> the object's type
 type(name, bases, dict) -> a new type""")
     w_type = _check(space, w_type)
     if not w_type.is_heaptype():
@@ -894,7 +957,7 @@ def descr__flags(space, w_type):
         flags |= _CPYTYPE
     if w_type.flag_abstract:
         flags |= _ABSTRACT
-    return space.wrap(flags)
+    return space.newint(flags)
 
 def descr_get__module(space, w_type):
     w_type = _check(space, w_type)
@@ -957,7 +1020,7 @@ W_TypeObject.typedef = TypeDef("type",
     __base__ = GetSetProperty(descr__base),
     __mro__ = GetSetProperty(descr_get__mro__),
     __dict__=GetSetProperty(type_get_dict),
-    __doc__ = GetSetProperty(descr__doc),
+    __doc__ = GetSetProperty(descr__doc, cls=W_TypeObject, name='__doc__'),
     mro = gateway.interp2app(descr_mro),
     __flags__ = GetSetProperty(descr__flags),
     __module__ = GetSetProperty(descr_get__module, descr_set__module),
@@ -1035,7 +1098,7 @@ def copy_flags_from_bases(w_self, w_bestbase):
 
 
 def create_all_slots(w_self, hasoldstylebase, w_bestbase, force_new_layout):
-    from pypy.objspace.std.listobject import StringSort
+    from pypy.interpreter.miscutils import string_sort
 
     base_layout = w_bestbase.layout
     index_next_extra_slot = base_layout.nslots
@@ -1049,13 +1112,13 @@ def create_all_slots(w_self, hasoldstylebase, w_bestbase, force_new_layout):
         wantdict = False
         wantweakref = False
         w_slots = dict_w['__slots__']
-        if (space.isinstance_w(w_slots, space.w_str) or
+        if (space.isinstance_w(w_slots, space.w_bytes) or
             space.isinstance_w(w_slots, space.w_unicode)):
             slot_names_w = [w_slots]
         else:
             slot_names_w = space.unpackiterable(w_slots)
         for w_slot_name in slot_names_w:
-            slot_name = space.str_w(w_slot_name)
+            slot_name = space.text_w(w_slot_name)
             if slot_name == '__dict__':
                 if wantdict or w_bestbase.hasdict:
                     raise oefmt(space.w_TypeError,
@@ -1069,8 +1132,7 @@ def create_all_slots(w_self, hasoldstylebase, w_bestbase, force_new_layout):
             else:
                 newslotnames.append(slot_name)
         # Sort the list of names collected so far
-        sorter = StringSort(newslotnames, len(newslotnames))
-        sorter.sort()
+        string_sort(newslotnames)
         # Try to create all slots in order.  The creation of some of
         # them might silently fail; then we delete the name from the
         # list.  At the end, 'index_next_extra_slot' has been advanced
@@ -1106,24 +1168,24 @@ def create_slot(w_self, slot_name, index_next_extra_slot):
     slot_name = mangle(slot_name, w_self.name)
     if slot_name not in w_self.dict_w:
         # Force interning of slot names.
-        slot_name = space.str_w(space.new_interned_str(slot_name))
+        slot_name = space.text_w(space.new_interned_str(slot_name))
         # in cpython it is ignored less, but we probably don't care
         member = Member(index_next_extra_slot, slot_name, w_self)
-        w_self.dict_w[slot_name] = space.wrap(member)
+        w_self.dict_w[slot_name] = member
         return True
     else:
         return False
 
 def create_dict_slot(w_self):
     if not w_self.hasdict:
-        w_self.dict_w.setdefault('__dict__',
-                                 w_self.space.wrap(dict_descr))
+        descr = dict_descr.copy_for_type(w_self)
+        w_self.dict_w.setdefault('__dict__', descr)
         w_self.hasdict = True
 
 def create_weakref_slot(w_self):
     if not w_self.weakrefable:
-        w_self.dict_w.setdefault('__weakref__',
-                                 w_self.space.wrap(weakref_descr))
+        descr = weakref_descr.copy_for_type(w_self)
+        w_self.dict_w.setdefault('__weakref__', descr)
         w_self.weakrefable = True
 
 def valid_slot_name(slot_name):
@@ -1138,7 +1200,6 @@ def setup_user_defined_type(w_self, force_new_layout):
     if len(w_self.bases_w) == 0:
         w_self.bases_w = [w_self.space.w_object]
     w_bestbase = check_and_find_best_base(w_self.space, w_self.bases_w)
-    w_self.flag_heaptype = True
     for w_base in w_self.bases_w:
         if not isinstance(w_base, W_TypeObject):
             continue
@@ -1157,9 +1218,12 @@ def setup_user_defined_type(w_self, force_new_layout):
 def setup_builtin_type(w_self, instancetypedef):
     w_self.hasdict = instancetypedef.hasdict
     w_self.weakrefable = instancetypedef.weakrefable
-    w_self.w_doc = w_self.space.wrap(instancetypedef.doc)
+    if isinstance(instancetypedef.doc, W_Root):
+        w_doc = instancetypedef.doc
+    else:
+        w_doc = w_self.space.newtext_or_none(instancetypedef.doc)
+    w_self.w_doc = w_doc
     ensure_common_attributes(w_self)
-    w_self.flag_heaptype = instancetypedef.heaptype
     #
     # usually 'instancetypedef' is new, i.e. not seen in any base,
     # but not always (see Exception class)
@@ -1195,7 +1259,7 @@ def ensure_module_attr(w_self):
         caller = space.getexecutioncontext().gettopframe_nohidden()
         if caller is not None:
             w_globals = caller.get_w_globals()
-            w_name = space.finditem(w_globals, space.wrap('__name__'))
+            w_name = space.finditem(w_globals, space.newtext('__name__'))
             if w_name is not None:
                 w_self.dict_w['__module__'] = w_name
 
@@ -1308,13 +1372,17 @@ def mro_error(space, orderlists):
 
 
 class TypeCache(SpaceCache):
+    @not_rpython
     def build(self, typedef):
-        "NOT_RPYTHON: initialization-time only."
+        "initialization-time only."
         from pypy.objspace.std.objectobject import W_ObjectObject
+        from pypy.interpreter.typedef import GetSetProperty
+        from rpython.rlib.objectmodel import instantiate
 
         space = self.space
         rawdict = typedef.rawdict
         lazyloaders = {}
+        w_type = instantiate(W_TypeObject)
 
         # compute the bases
         if typedef is W_ObjectObject.typedef:
@@ -1326,16 +1394,20 @@ class TypeCache(SpaceCache):
         # wrap everything
         dict_w = {}
         for descrname, descrvalue in rawdict.items():
+            # special case for GetSetProperties' __objclass__:
+            if isinstance(descrvalue, GetSetProperty):
+                descrvalue = descrvalue.copy_for_type(w_type)
             dict_w[descrname] = space.wrap(descrvalue)
 
         if typedef.applevel_subclasses_base is not None:
             overridetypedef = typedef.applevel_subclasses_base.typedef
         else:
             overridetypedef = typedef
-        w_type = W_TypeObject(space, typedef.name, bases_w, dict_w,
-                              overridetypedef=overridetypedef)
+        w_type.__init__(space, typedef.name, bases_w, dict_w,
+                              overridetypedef=overridetypedef,
+                              is_heaptype=overridetypedef.heaptype)
         if typedef is not overridetypedef:
-            w_type.w_doc = space.wrap(typedef.doc)
+            w_type.w_doc = space.newtext_or_none(typedef.doc)
         if hasattr(typedef, 'flag_sequence_bug_compat'):
             w_type.flag_sequence_bug_compat = typedef.flag_sequence_bug_compat
         w_type.lazyloaders = lazyloaders
