@@ -14,7 +14,6 @@ MODEL_X86         = 'x86'
 MODEL_X86_NO_SSE2 = 'x86-without-sse2'
 MODEL_X86_64      = 'x86-64'
 MODEL_ARM         = 'arm'
-MODEL_ARM64       = 'aarch64'
 MODEL_PPC_64      = 'ppc-64'
 MODEL_S390_64     = 's390x'
 # don't use '_' in the model strings; they are replaced by '-'
@@ -26,7 +25,6 @@ def detect_model_from_c_compiler():
     mapping = {
         MODEL_X86_64: ['__amd64__', '__amd64', '__x86_64__', '__x86_64', '_M_X64', '_M_AMD64'],
         MODEL_ARM:    ['__arm__', '__thumb__','_M_ARM_EP'],
-        MODEL_ARM64:  ['__aarch64__'],
         MODEL_X86:    ['i386', '__i386', '__i386__', '__i686__','_M_IX86'],
         MODEL_PPC_64: ['__powerpc64__'],
         MODEL_S390_64:['__s390x__'],
@@ -70,8 +68,6 @@ def detect_model_from_host_platform():
             'amd64': MODEL_X86,    # freebsd
             'AMD64': MODEL_X86,    # win64
             'armv8l': MODEL_ARM,   # 32-bit ARMv8
-            'aarch64': MODEL_ARM64,
-            'arm64': MODEL_ARM64,
             'armv7l': MODEL_ARM,
             'armv6l': MODEL_ARM,
             'arm': MODEL_ARM,      # freebsd
@@ -123,8 +119,6 @@ def getcpuclassname(backend_name="auto"):
         return "rpython.jit.backend.x86.runner", "CPU_X86_64"
     elif backend_name == MODEL_ARM:
         return "rpython.jit.backend.arm.runner", "CPU_ARM"
-    elif backend_name == MODEL_ARM64:
-        return "rpython.jit.backend.aarch64.runner", "CPU_ARM64"
     elif backend_name == MODEL_PPC_64:
         return "rpython.jit.backend.ppc.runner", "PPC_CPU"
     elif backend_name == MODEL_S390_64:
@@ -147,7 +141,6 @@ def getcpufeatures(backend_name="auto"):
         MODEL_X86_NO_SSE2: ['longlong'],
         MODEL_X86_64: ['floats', 'singlefloats'],
         MODEL_ARM: ['floats', 'singlefloats', 'longlong'],
-        MODEL_ARM64: ['floats'],
         MODEL_PPC_64: ['floats'],
         MODEL_S390_64: ['floats'],
     }[backend_name]
