@@ -10,8 +10,7 @@ from rpython.jit.metainterp.optimizeopt.optimizer import Optimizer,\
 from rpython.jit.metainterp.optimizeopt.vstring import StrPtrInfo
 from rpython.jit.metainterp.optimizeopt.virtualstate import (
     VirtualStateConstructor, VirtualStatesCantMatch)
-from rpython.jit.metainterp.resoperation import rop, ResOperation, GuardResOp,\
-     AbstractResOp
+from rpython.jit.metainterp.resoperation import rop, ResOperation, GuardResOp
 from rpython.jit.metainterp import compile
 from rpython.rlib.debug import debug_print, debug_start, debug_stop,\
      have_debug_prints
@@ -22,7 +21,6 @@ class UnrollableOptimizer(Optimizer):
             if self.optunroll.short_preamble_producer is None:
                 assert False # unreachable code
             op = preamble_op.op
-            self.optimizer.inparg_dict[op] = None # XXX ARGH
             # special hack for int_add(x, accumulator-const) optimization
             self.optunroll.short_preamble_producer.use_box(op,
                                                 preamble_op.preamble_op, self)
@@ -86,12 +84,14 @@ class UnrollableOptimizer(Optimizer):
             if preamble_info.is_nonnull():
                 self.make_nonnull(op)
         elif isinstance(preamble_info, intutils.IntBound):
-            if preamble_info.lower > MININT/2 or preamble_info.upper < MAXINT/2:
+            fix_lo = preamble_info.has_lower and preamble_info.lower >= MININT/2
+            fix_up = preamble_info.has_upper and preamble_info.upper <= MAXINT/2
+            if fix_lo or fix_up:
                 intbound = self.getintbound(op)
-                if preamble_info.has_lower and preamble_info.lower > MININT/2:
+                if fix_lo:
                     intbound.has_lower = True
                     intbound.lower = preamble_info.lower
-                if preamble_info.has_upper and preamble_info.upper < MAXINT/2:
+                if fix_up:
                     intbound.has_upper = True
                     intbound.upper = preamble_info.upper
         elif isinstance(preamble_info, info.FloatConstInfo):
@@ -142,7 +142,6 @@ class UnrollOptimizer(Optimization):
         except VirtualStatesCantMatch:
             raise InvalidLoop("Cannot import state, virtual states don't match")
         self.potential_extra_ops = {}
-        self.optimizer.init_inparg_dict_from(label_args)
         try:
             info, _ = self.optimizer.propagate_all_forward(
                 trace, call_pure_results, flush=False)
@@ -237,9 +236,15 @@ class UnrollOptimizer(Optimization):
         return label_vs
 
     def optimize_bridge(self, trace, runtime_boxes, call_pure_results,
-                        inline_short_preamble, box_names_memo):
+                        inline_short_preamble, box_names_memo, resumestorage):
+        from rpython.jit.metainterp.optimizeopt.bridgeopt import deserialize_optimizer_knowledge
+        frontend_inputargs = trace.inputargs
         trace = trace.get_iter()
         self._check_no_forwarding([trace.inputargs])
+        if resumestorage:
+            deserialize_optimizer_knowledge(self.optimizer,
+                                            resumestorage, frontend_inputargs,
+                                            trace.inputargs)
         info, ops = self.optimizer.propagate_all_forward(trace,
             call_pure_results, False)
         jump_op = info.jump_op
@@ -411,7 +416,8 @@ class UnrollOptimizer(Optimization):
                 i += 1
                 self.optimizer.send_extra_operation(op)
             # force all of them except the virtuals
-            for arg in args_no_virtuals + short_jump_args:
+            for arg in (args_no_virtuals +
+                        self._map_args(mapping, short_jump_args)):
                 self.optimizer.force_box(self.get_box_replacement(arg))
             self.optimizer.flush()
             # done unless "short" has grown again
@@ -422,8 +428,9 @@ class UnrollOptimizer(Optimization):
                 for box in self._map_args(mapping, short_jump_args)]
 
     def _expand_info(self, arg, infos):
-        if isinstance(arg, AbstractResOp) and rop.is_same_as(arg.opnum):
-            info = self.optimizer.getinfo(arg.getarg(0))
+        arg1 = self.optimizer.as_operation(arg)
+        if arg1 is not None and rop.is_same_as(arg1.opnum):
+            info = self.optimizer.getinfo(arg1.getarg(0))
         else:
             info = self.optimizer.getinfo(arg)
         if arg in infos:
