@@ -7,49 +7,25 @@ from rpython.annotator.model import (SomeObject, SomeString, s_None, SomeChar,
 from rpython.rtyper.llannotation import SomePtr
 from rpython.rlib import jit
 from rpython.rlib.objectmodel import newlist_hint, resizelist_hint, specialize, not_rpython
-from rpython.rlib.rarithmetic import ovfcheck, LONG_BIT as BLOOM_WIDTH, intmask
+from rpython.rlib.rarithmetic import ovfcheck, LONG_BIT as BLOOM_WIDTH
+from rpython.rlib.unicodedata import unicodedb_5_2_0 as unicodedb
 from rpython.rtyper.extregistry import ExtRegistryEntry
 from rpython.tool.pairtype import pairtype
 
 
 # -------------- public API for string functions -----------------------
 
-@specialize.ll_and_arg(2)
-def _isspace(s, pos, isutf8=False):
-    from rpython.rlib.unicodedata import unicodedb_5_2_0 as unicodedb
-    if isutf8:
-        from rpython.rlib import rutf8
-        return rutf8.isspace(s, pos)
-    char = s[pos]
+@specialize.argtype(0)
+def _isspace(char):
     if isinstance(char, str):
         return char.isspace()
     else:
         assert isinstance(char, unicode)
         return unicodedb.isspace(ord(char))
 
-@specialize.ll_and_arg(2)
-def _incr(s, pos, isutf8):
-    if isutf8:
-        from rpython.rlib.rutf8 import next_codepoint_pos
-        assert pos >= 0
-        r = next_codepoint_pos(s, pos)
-        assert r >= 0
-        return r
-    else:
-        return pos + 1
 
-@specialize.ll_and_arg(2)
-def _decr(s, pos, isutf8):
-    if isutf8:
-        from rpython.rlib.rutf8 import prev_codepoint_pos
-        if pos <= 0:
-            return -1
-        return intmask(prev_codepoint_pos(s, pos))
-    else:
-        return pos - 1
-
-@specialize.ll_and_arg(3)
-def split(value, by=None, maxsplit=-1, isutf8=False):
+@specialize.argtype(0, 1)
+def split(value, by=None, maxsplit=-1):
     if by is None:
         length = len(value)
         i = 0
@@ -57,9 +33,9 @@ def split(value, by=None, maxsplit=-1, isutf8=False):
         while True:
             # find the beginning of the next word
             while i < length:
-                if not _isspace(value, i, isutf8):
+                if not _isspace(value[i]):
                     break   # found
-                i = _incr(value, i, isutf8)
+                i += 1
             else:
                 break  # end of string, finished
 
@@ -67,25 +43,18 @@ def split(value, by=None, maxsplit=-1, isutf8=False):
             if maxsplit == 0:
                 j = length   # take all the rest of the string
             else:
-                j = _incr(value, i, isutf8)
-                while j < length and not _isspace(value, j, isutf8):
-                    j = _incr(value, j, isutf8)
+                j = i + 1
+                while j < length and not _isspace(value[j]):
+                    j += 1
                 maxsplit -= 1   # NB. if it's already < 0, it stays < 0
 
             # the word is value[i:j]
             res.append(value[i:j])
 
             # continue to look from the character following the space after the word
-            if j < length:
-                i = _incr(value, j, isutf8)
-            else:
-                break
+            i = j + 1
         return res
-    else:
-        return _split_by(value, by, maxsplit)
 
-@specialize.argtype(0)
-def _split_by(value, by, maxsplit):
     if isinstance(value, unicode):
         assert isinstance(by, unicode)
     if isinstance(value, str):
@@ -96,18 +65,12 @@ def _split_by(value, by, maxsplit):
     bylen = len(by)
     if bylen == 0:
         raise ValueError("empty separator")
-    # XXX measure if preallocating the result list to the correct
-    #     size is faster, should be
 
     start = 0
     if bylen == 1:
-        # fast path: uses str.find(character) and str.count(character)
+        # fast path: uses str.rfind(character) and str.count(character)
         by = by[0]    # annotator hack: string -> char
         cnt = count(value, by, 0, len(value))
-        if cnt == 0:
-            if isinstance(value, str):
-                return [value]
-            return [value[0:len(value)]]
         if 0 <= maxsplit < cnt:
             cnt = maxsplit
         res = newlist_hint(cnt + 1)
@@ -138,49 +101,42 @@ def _split_by(value, by, maxsplit):
     return res
 
 
-@specialize.ll_and_arg(3)
-def rsplit(value, by=None, maxsplit=-1, isutf8=False):
+@specialize.argtype(0, 1)
+def rsplit(value, by=None, maxsplit=-1):
     if by is None:
         res = []
 
-        i = _decr(value, len(value), isutf8)
+        i = len(value) - 1
         while True:
             # starting from the end, find the end of the next word
             while i >= 0:
-                if not _isspace(value, i, isutf8):
+                if not _isspace(value[i]):
                     break   # found
-                i = _decr(value, i, isutf8)
+                i -= 1
             else:
                 break  # end of string, finished
 
-            # find the start of the word as 'j1'
+            # find the start of the word
+            # (more precisely, 'j' will be the space character before the word)
             if maxsplit == 0:
-                j1 = 0   # take all the rest of the string
-                j = -1
+                j = -1   # take all the rest of the string
             else:
-                j1 = i
-                while True:
-                    j = _decr(value, j1, isutf8)
-                    if j < 0 or _isspace(value, j, isutf8):
-                        break
-                    j1 = j
+                j = i - 1
+                while j >= 0 and not _isspace(value[j]):
+                    j -= 1
                 maxsplit -= 1   # NB. if it's already < 0, it stays < 0
 
-            # the word is value[j1:i+1]
+            # the word is value[j+1:i+1]
+            j1 = j + 1
             assert j1 >= 0
-            i1 = _incr(value, i, isutf8)
-            res.append(value[j1:i1])
+            res.append(value[j1:i+1])
 
             # continue to look from the character before the space before the word
-            i = _decr(value, j, isutf8)
+            i = j - 1
 
         res.reverse()
         return res
-    else:
-        return _rsplit_by(value, by, maxsplit)
 
-@specialize.argtype(0)
-def _rsplit_by(value, by, maxsplit):
     if isinstance(value, unicode):
         assert isinstance(by, unicode)
     if isinstance(value, str):
@@ -212,12 +168,7 @@ def _rsplit_by(value, by, maxsplit):
 
 @specialize.argtype(0, 1)
 @jit.elidable
-def replace(input, sub, by, maxcount=-1):
-    return replace_count(input, sub, by, maxcount)[0]
-
-@specialize.ll_and_arg(4)
-@jit.elidable
-def replace_count(input, sub, by, maxcount=-1, isutf8=False):
+def replace(input, sub, by, maxsplit=-1):
     if isinstance(input, str):
         Builder = StringBuilder
     elif isinstance(input, unicode):
@@ -225,14 +176,14 @@ def replace_count(input, sub, by, maxcount=-1, isutf8=False):
     else:
         assert isinstance(input, list)
         Builder = ByteListBuilder
-    if maxcount == 0:
-        return input, 0
+    if maxsplit == 0:
+        return input
 
 
-    if not sub and not isutf8:
+    if not sub:
         upper = len(input)
-        if maxcount > 0 and maxcount < upper + 2:
-            upper = maxcount - 1
+        if maxsplit > 0 and maxsplit < upper + 2:
+            upper = maxsplit - 1
             assert upper >= 0
 
         try:
@@ -249,160 +200,34 @@ def replace_count(input, sub, by, maxcount=-1, isutf8=False):
             builder.append(input[i])
         builder.append(by)
         builder.append_slice(input, upper, len(input))
-        replacements = upper + 1
-
-    elif isinstance(input, str) and len(sub) == 1:
-        if len(by) == 1:
-            return replace_count_str_chr_chr(input, sub[0], by[0], maxcount)
-        return replace_count_str_chr_str(input, sub[0], by, maxcount)
-
     else:
         # First compute the exact result size
-        if sub:
-            cnt = count(input, sub, 0, len(input))
-            if isinstance(input, str) and cnt == 0:
-                return input, 0
-            if isinstance(input, str):
-                return replace_count_str_str_str(input, sub, by, cnt, maxcount)
-        else:
-            assert isutf8
-            from rpython.rlib import rutf8
-            cnt = rutf8.codepoints_in_utf8(input) + 1
-
-        if cnt > maxcount and maxcount > 0:
-            cnt = maxcount
+        cnt = count(input, sub, 0, len(input))
+        if cnt > maxsplit and maxsplit > 0:
+            cnt = maxsplit
         diff_len = len(by) - len(sub)
         try:
             result_size = ovfcheck(diff_len * cnt)
             result_size = ovfcheck(result_size + len(input))
         except OverflowError:
             raise
-        replacements = cnt
 
         builder = Builder(result_size)
         start = 0
         sublen = len(sub)
 
-        if sublen == 0:
-            assert isutf8
-            from rpython.rlib import rutf8
-            while True:
-                builder.append(by)
-                maxcount -= 1
-                if start == len(input) or maxcount == 0:
-                    break
-                next = rutf8.next_codepoint_pos(input, start)
-                builder.append_slice(input, start, next)
-                start = next
-        else:
-            while maxcount != 0:
-                next = find(input, sub, start, len(input))
-                if next < 0:
-                    break
-                builder.append_slice(input, start, next)
-                builder.append(by)
-                start = next + sublen
-                maxcount -= 1   # NB. if it's already < 0, it stays < 0
+        while maxsplit != 0:
+            next = find(input, sub, start, len(input))
+            if next < 0:
+                break
+            builder.append_slice(input, start, next)
+            builder.append(by)
+            start = next + sublen
+            maxsplit -= 1   # NB. if it's already < 0, it stays < 0
 
         builder.append_slice(input, start, len(input))
 
-    return builder.build(), replacements
-
-def replace_count_str_chr_chr(input, c1, c2, maxcount):
-    from rpython.rtyper.annlowlevel import llstr, hlstr
-    s = llstr(input)
-    length = len(s.chars)
-    start = find(input, c1, 0, len(input))
-    if start < 0:
-        return input, 0
-    newstr = s.malloc(length)
-    src = s.chars
-    dst = newstr.chars
-    s.copy_contents(s, newstr, 0, 0, len(input))
-    dst[start] = c2
-    count = 1
-    start += 1
-    maxcount -= 1
-    while maxcount != 0:
-        next = find(input, c1, start, len(input))
-        if next < 0:
-            break
-        dst[next] = c2
-        start = next + 1
-        maxcount -= 1
-        count += 1
-
-    return hlstr(newstr), count
-
-def replace_count_str_chr_str(input, sub, by, maxcount):
-    from rpython.rtyper.annlowlevel import llstr, hlstr
-    cnt = count(input, sub, 0, len(input))
-    if cnt == 0:
-        return input, 0
-    if maxcount > 0 and cnt > maxcount:
-        cnt = maxcount
-    diff_len = len(by) - 1
-    try:
-        result_size = ovfcheck(diff_len * cnt)
-        result_size = ovfcheck(result_size + len(input))
-    except OverflowError:
-        raise
-
-    s = llstr(input)
-    by_ll = llstr(by)
-
-    newstr = s.malloc(result_size)
-    dst = 0
-    start = 0
-    while maxcount != 0:
-        next = find(input, sub, start, len(input))
-        if next < 0:
-            break
-        s.copy_contents(s, newstr, start, dst, next - start)
-        dst += next - start
-        s.copy_contents(by_ll, newstr, 0, dst, len(by))
-        dst += len(by)
-
-        start = next + 1
-        maxcount -= 1   # NB. if it's already < 0, it stays < 0
-
-    s.copy_contents(s, newstr, start, dst, len(input) - start)
-    assert dst - start + len(input) == result_size
-    return hlstr(newstr), cnt
-
-def replace_count_str_str_str(input, sub, by, cnt, maxcount):
-    from rpython.rtyper.annlowlevel import llstr, hlstr
-    if cnt > maxcount and maxcount > 0:
-        cnt = maxcount
-    diff_len = len(by) - len(sub)
-    try:
-        result_size = ovfcheck(diff_len * cnt)
-        result_size = ovfcheck(result_size + len(input))
-    except OverflowError:
-        raise
-
-    s = llstr(input)
-    by_ll = llstr(by)
-    newstr = s.malloc(result_size)
-    sublen = len(sub)
-    bylen = len(by)
-    inputlen = len(input)
-    dst = 0
-    start = 0
-    while maxcount != 0:
-        next = find(input, sub, start, inputlen)
-        if next < 0:
-            break
-        s.copy_contents(s, newstr, start, dst, next - start)
-        dst += next - start
-        s.copy_contents(by_ll, newstr, 0, dst, bylen)
-        dst += bylen
-        start = next + sublen
-        maxcount -= 1   # NB. if it's already < 0, it stays < 0
-    s.copy_contents(s, newstr, start, dst, len(input) - start)
-    assert dst - start + len(input) == result_size
-    return hlstr(newstr), cnt
-
+    return builder.build()
 
 def _normalize_start_end(length, start, end):
     if start < 0:
@@ -465,37 +290,20 @@ def count(value, other, start, end):
     return _search(value, other, start, end, SEARCH_COUNT)
 
 # -------------- substring searching helper ----------------
+# XXX a lot of code duplication with lltypesystem.rstr :-(
 
 SEARCH_COUNT = 0
 SEARCH_FIND = 1
 SEARCH_RFIND = 2
 
-@specialize.ll()
 def bloom_add(mask, c):
     return mask | (1 << (ord(c) & (BLOOM_WIDTH - 1)))
 
-@specialize.ll()
 def bloom(mask, c):
     return mask & (1 << (ord(c) & (BLOOM_WIDTH - 1)))
 
 @specialize.argtype(0, 1)
 def _search(value, other, start, end, mode):
-    if isinstance(value, str) and isinstance(other, str):
-        return _search_elidable(value, other, start, end, mode)
-    return _search_normal(value, other, start, end, mode)
-
-@specialize.argtype(0, 1)
-@jit.elidable
-def _search_elidable(value, other, start, end, mode):
-    return _search_normal(value, other, start, end, mode)
-
-@specialize.argtype(0, 1)
-def _search_normal(value, other, start, end, mode):
-    assert value is not None
-    if isinstance(value, unicode):
-        NUL = u'\0'
-    else:
-        NUL = '\0'
     if start < 0:
         start = 0
     if end > len(value):
@@ -525,7 +333,7 @@ def _search_normal(value, other, start, end, mode):
         return -1
 
     mlast = m - 1
-    skip = mlast
+    skip = mlast - 1
     mask = 0
 
     if mode != SEARCH_RFIND:
@@ -538,7 +346,7 @@ def _search_normal(value, other, start, end, mode):
         i = start - 1
         while i + 1 <= start + w:
             i += 1
-            if value[i + mlast] == other[mlast]:
+            if value[i + m - 1] == other[m - 1]:
                 for j in range(mlast):
                     if value[i + j] != other[j]:
                         break
@@ -552,7 +360,7 @@ def _search_normal(value, other, start, end, mode):
                 if i + m < len(value):
                     c = value[i + m]
                 else:
-                    c = NUL
+                    c = '\0'
                 if not bloom(mask, c):
                     i += m
                 else:
@@ -561,7 +369,7 @@ def _search_normal(value, other, start, end, mode):
                 if i + m < len(value):
                     c = value[i + m]
                 else:
-                    c = NUL
+                    c = '\0'
                 if not bloom(mask, c):
                     i += m
     else:
@@ -619,10 +427,6 @@ class ParseStringOverflowError(Exception):
     def __init__(self, parser):
         self.parser = parser
 
-class MaxDigitsError(Exception):
-    def __init__(self, digits):
-        self.digits = digits
-
 # iterator-like class
 class NumberStringParser:
 
@@ -631,94 +435,60 @@ class NumberStringParser:
                                (self.fname, self.original_base))
 
     def __init__(self, s, literal, base, fname, allow_underscores=False,
-                 no_implicit_octal=False, start=0, end=-1, max_str_digits=0):
+                 no_implicit_octal=False):
         self.fname = fname
         sign = 1
-        self.s = s
-        self.start = start
-        if end == -1:
-            end = len(s)
-        self.end = end
-        self._strip_spaces()
-        if self._startswith1('-'):
+        if s.startswith('-'):
             sign = -1
-            self.start += 1
-            self._strip_spaces()
-        elif self._startswith1('+'):
-            self.start += 1
-            self._strip_spaces()
+            s = strip_spaces(s[1:])
+        elif s.startswith('+'):
+            s = strip_spaces(s[1:])
         self.sign = sign
         self.original_base = base
         self.allow_underscores = allow_underscores
 
         if base == 0:
-            if self._startswith2('0x') or self._startswith2('0X'):
+            if s.startswith('0x') or s.startswith('0X'):
                 base = 16
-            elif self._startswith2('0b') or self._startswith2('0B'):
+            elif s.startswith('0b') or s.startswith('0B'):
                 base = 2
-            elif self._startswith1('0'): # also covers the '0o' case
-                if no_implicit_octal and not (self._startswith2('0o') or
-                                              self._startswith2('0O')):
+            elif s.startswith('0'): # also covers the '0o' case
+                if no_implicit_octal and not (s.startswith('0o') or
+                                              s.startswith('0O')):
                     base = 1    # this makes only the digit '0' valid...
                 else:
                     base = 8
             else:
                 base = 10
         elif base < 2 or base > 36:
-            raise InvalidBaseError("%s() base must be >= 2 and <= 36, or 0" % fname)
+            raise InvalidBaseError("%s() base must be >= 2 and <= 36" % fname)
         self.base = base
 
         # Leading underscores are not allowed
-        if self._startswith1('_'):
+        if s.startswith('_'):
             self.error()
 
-        if base == 16 and (self._startswith2('0x') or self._startswith2('0X')):
-            self.start += 2
-        if base == 8 and (self._startswith2('0o') or self._startswith2('0O')):
-            self.start += 2
-        if base == 2 and (self._startswith2('0b') or self._startswith2('0B')):
-            self.start += 2
-        if self.start == self.end:
+        if base == 16 and (s.startswith('0x') or s.startswith('0X')):
+            s = s[2:]
+        if base == 8 and (s.startswith('0o') or s.startswith('0O')):
+            s = s[2:]
+        if base == 2 and (s.startswith('0b') or s.startswith('0B')):
+            s = s[2:]
+        if not s:
             self.error()
-        self.i = self.start
-        if max_str_digits > 0:
-            length =  self.end - self.start - self.s.count('_')
-            if length > max_str_digits:
-                raise MaxDigitsError(length)
-
-    def _startswith1(self, prefix):
-        if self.start >= self.end:
-            return False
-        return self.s[self.start] == prefix[0]
-        return startswith(self.s, prefix, start=self.start, end=self.end)
-
-    def _startswith2(self, prefix):
-        if self.start + 1 >= self.end:
-            return False
-        return self.s[self.start] == prefix[0] and self.s[self.start + 1] == prefix[1]
-
-    def _strip_spaces(self):
-        # XXX this is not locale-dependent
-        p = self.start
-        q = self.end
-        s = self.s
-        while p < q and s[p] in ' \f\n\r\t\v':
-            p += 1
-        while p < q and s[q-1] in ' \f\n\r\t\v':
-            q -= 1
-        assert q >= p
-        self.start = p
-        self.end = q
+        self.s = s
+        self.n = len(s)
+        self.i = 0
 
     def rewind(self):
-        self.i = self.start
+        self.i = 0
 
     def next_digit(self): # -1 => exhausted
-        if self.i < self.end:
+        if self.i < self.n:
             c = self.s[self.i]
             if self.allow_underscores and c == '_':
                 self.i += 1
-                if self.i >= self.end:
+                if self.i >= self.n:
                     self.error()
                 c = self.s[self.i]
             digit = ord(c)
@@ -741,14 +511,9 @@ class NumberStringParser:
         # After exhausting all n digits in next_digit(), you can walk them
         # again in reverse order by calling prev_digit() exactly n times
         i = self.i - 1
-        assert i >= self.start
+        assert i >= 0
         self.i = i
         c = self.s[i]
-        if self.allow_underscores and c == '_':
-            i = self.i - 1
-            assert i >= 0
-            self.i = i
-            c = self.s[i]
         digit = ord(c)
         if '0' <= c <= '9':
             digit -= ord('0')
@@ -876,7 +641,7 @@ class SomeStringBuilder(SomeObject):
         return s_None
 
     def method_append_multiple_char(self, s_char, s_times):
-        assert isinstance(s_char, (SomeString, SomeChar))
+        assert isinstance(s_char, SomeChar)
         assert isinstance(s_times, SomeInteger)
         return s_None
 
