@@ -26,7 +26,6 @@ REG_NOTIFY_CHANGE_LAST_SET REG_NOTIFY_CHANGE_SECURITY REG_LEGAL_CHANGE_FILTER
 REG_NONE REG_SZ REG_EXPAND_SZ REG_BINARY REG_DWORD REG_DWORD_LITTLE_ENDIAN
 REG_DWORD_BIG_ENDIAN REG_LINK REG_MULTI_SZ REG_RESOURCE_LIST
 REG_FULL_RESOURCE_DESCRIPTOR REG_RESOURCE_REQUIREMENTS_LIST
-REG_QWORD REG_QWORD_LITTLE_ENDIAN
 
 HKEY_LOCAL_MACHINE HKEY_CLASSES_ROOT HKEY_CURRENT_CONFIG HKEY_CURRENT_USER
 HKEY_DYN_DATA HKEY_LOCAL_MACHINE HKEY_PERFORMANCE_DATA HKEY_USERS
@@ -50,75 +49,71 @@ PHKEY = rffi.CArrayPtr(HKEY)
 REGSAM = rwin32.DWORD
 
 def get_traits(suffix):
-    if suffix == 'A':
-        strp = rffi.CCHARP
-    else:
-        strp = rffi.CWCHARP
     RegSetValue = external(
         'RegSetValue' + suffix,
-        [HKEY, strp, rwin32.DWORD, strp, rwin32.DWORD],
+        [HKEY, rffi.CCHARP, rwin32.DWORD, rffi.CCHARP, rwin32.DWORD],
         rffi.LONG)
 
     RegSetValueEx = external(
         'RegSetValueEx' + suffix,
-        [HKEY, strp, rwin32.DWORD,
-         rwin32.DWORD, strp, rwin32.DWORD],
+        [HKEY, rffi.CCHARP, rwin32.DWORD,
+         rwin32.DWORD, rffi.CCHARP, rwin32.DWORD],
         rffi.LONG)
 
     RegQueryValue = external(
         'RegQueryValue' + suffix,
-        [HKEY, strp, strp, rwin32.PLONG],
+        [HKEY, rffi.CCHARP, rffi.CCHARP, rwin32.PLONG],
         rffi.LONG)
 
     RegQueryValueEx = external(
         'RegQueryValueEx' + suffix,
-        [HKEY, strp, rwin32.LPDWORD, rwin32.LPDWORD,
-         strp, rwin32.LPDWORD],
+        [HKEY, rffi.CCHARP, rwin32.LPDWORD, rwin32.LPDWORD,
+         rffi.CCHARP, rwin32.LPDWORD],
         rffi.LONG)
 
     RegCreateKey = external(
         'RegCreateKey' + suffix,
-        [HKEY, strp, PHKEY],
+        [HKEY, rffi.CCHARP, PHKEY],
         rffi.LONG)
 
     RegCreateKeyEx = external(
         'RegCreateKeyEx' + suffix,
-        [HKEY, strp, rwin32.DWORD, strp, rwin32.DWORD,
+        [HKEY, rffi.CCHARP, rwin32.DWORD, rffi.CCHARP, rwin32.DWORD,
          REGSAM, rffi.VOIDP, PHKEY, rwin32.LPDWORD],
         rffi.LONG)
 
     RegDeleteValue = external(
         'RegDeleteValue' + suffix,
-        [HKEY, strp],
+        [HKEY, rffi.CCHARP],
         rffi.LONG)
 
     RegDeleteKey = external(
         'RegDeleteKey' + suffix,
-        [HKEY, strp],
+        [HKEY, rffi.CCHARP],
         rffi.LONG)
 
     RegOpenKeyEx = external(
         'RegOpenKeyEx' + suffix,
-        [HKEY, strp, rwin32.DWORD, REGSAM, PHKEY],
+        [HKEY, rffi.CCHARP, rwin32.DWORD, REGSAM, PHKEY],
         rffi.LONG)
 
     RegEnumValue = external(
         'RegEnumValue' + suffix,
-        [HKEY, rwin32.DWORD, strp,
+        [HKEY, rwin32.DWORD, rffi.CCHARP,
          rwin32.LPDWORD, rwin32.LPDWORD, rwin32.LPDWORD,
          rffi.CCHARP, rwin32.LPDWORD],
         rffi.LONG)
 
     RegEnumKeyEx = external(
         'RegEnumKeyEx' + suffix,
-        [HKEY, rwin32.DWORD, strp,
+        [HKEY, rwin32.DWORD, rffi.CCHARP,
          rwin32.LPDWORD, rwin32.LPDWORD,
-         strp, rwin32.LPDWORD, rwin32.PFILETIME],
+         rffi.CCHARP, rwin32.LPDWORD, rwin32.PFILETIME],
         rffi.LONG)
 
     RegQueryInfoKey = external(
         'RegQueryInfoKey' + suffix,
-        [HKEY, strp, rwin32.LPDWORD, rwin32.LPDWORD,
+        [HKEY, rffi.CCHARP, rwin32.LPDWORD, rwin32.LPDWORD,
          rwin32.LPDWORD, rwin32.LPDWORD, rwin32.LPDWORD,
          rwin32.LPDWORD, rwin32.LPDWORD, rwin32.LPDWORD,
          rwin32.LPDWORD, rwin32.PFILETIME],
@@ -126,17 +121,17 @@ def get_traits(suffix):
 
     RegLoadKey = external(
         'RegLoadKey' + suffix,
-        [HKEY, strp, strp],
+        [HKEY, rffi.CCHARP, rffi.CCHARP],
         rffi.LONG)
 
     RegSaveKey = external(
         'RegSaveKey' + suffix,
-        [HKEY, strp, rffi.VOIDP],
+        [HKEY, rffi.CCHARP, rffi.VOIDP],
         rffi.LONG)
 
     RegConnectRegistry = external(
         'RegConnectRegistry' + suffix,
-        [strp, HKEY, PHKEY],
+        [rffi.CCHARP, HKEY, PHKEY],
         rffi.LONG)
 
     return (RegSetValue, RegSetValueEx, RegQueryValue, RegQueryValueEx,
@@ -170,8 +165,8 @@ _ExpandEnvironmentStringsW = external(
     rwin32.DWORD,
     save_err=rffi.RFFI_SAVE_LASTERROR)
 
-def ExpandEnvironmentStrings(source, unicode_len):
-    with rffi.scoped_utf82wcharp(source, unicode_len) as src_buf:
+def ExpandEnvironmentStrings(source):
+    with rffi.scoped_unicode2wcharp(source) as src_buf:
         size = _ExpandEnvironmentStringsW(src_buf,
                                           lltype.nullptr(rffi.CWCHARP.TO), 0)
         if size == 0:
@@ -181,5 +176,4 @@ def ExpandEnvironmentStrings(source, unicode_len):
             if _ExpandEnvironmentStringsW(src_buf,
                                           dest_buf.raw, size) == 0:
                 raise rwin32.lastSavedWindowsError("ExpandEnvironmentStrings")
-            res = dest_buf.str(size-1) # remove trailing \0
-            return res.encode('utf8'), len(res)
+            return dest_buf.str(size - 1) # remove trailing \0
