@@ -25,12 +25,10 @@ class TestFile(BaseRtypingTest):
             f.close()
 
         f()
-        with open(fname, "r") as fid:
-            assert fid.read() == "dupa"
+        assert open(fname, "r").read() == "dupa"
         os.unlink(fname)
         self.interpret(f, [])
-        with open(fname, "r") as fid:
-            assert fid.read() == "dupa"
+        assert open(fname, "r").read() == "dupa"
 
     def test_open_errors(self):
         def f(run):
@@ -59,8 +57,8 @@ class TestFile(BaseRtypingTest):
                 assert False
 
             try:
-                rfile.create_fdopen_rfile(42, "badmode")
-            except  ValueError:
+                os.fdopen(42, "badmode")
+            except ValueError:
                 pass
             else:
                 assert False
@@ -90,7 +88,7 @@ class TestFile(BaseRtypingTest):
         f(sys.version_info >= (2, 7, 9))
         self.interpret(f, [True])
 
-    @py.test.mark.skipif("sys.platform == 'win32' or sys.platform == 'darwin'")
+    @py.test.mark.skipif("sys.platform == 'win32'")
     # http://msdn.microsoft.com/en-us/library/86cebhfs.aspx
     def test_open_buffering_line(self):
         fname = str(self.tmpdir.join('file_1a'))
@@ -108,7 +106,7 @@ class TestFile(BaseRtypingTest):
         os.unlink(fname)
         self.interpret(f, [])
 
-    @py.test.mark.skipif("sys.platform == 'win32' or sys.platform == 'darwin'")
+    @py.test.mark.skipif("sys.platform == 'win32'")
     # http://msdn.microsoft.com/en-us/library/86cebhfs.aspx
     def test_fdopen_buffering_line(self):
         fname = str(self.tmpdir.join('file_1a'))
@@ -128,7 +126,6 @@ class TestFile(BaseRtypingTest):
         os.unlink(fname)
         self.interpret(f, [])
 
-    @py.test.mark.skipif('sys.platform == "darwin"')
     def test_open_buffering_full(self):
         fname = str(self.tmpdir.join('file_1b'))
 
@@ -147,7 +144,6 @@ class TestFile(BaseRtypingTest):
         os.unlink(fname)
         self.interpret(f, [])
 
-    @py.test.mark.skipif('sys.platform == "darwin"')
     def test_fdopen_buffering_full(self):
         fname = str(self.tmpdir.join('file_1b'))
 
@@ -329,12 +325,10 @@ class TestFile(BaseRtypingTest):
             f2.close()
 
         f()
-        with open(fname) as fid:
-            assert fid.read() == "xxx"
+        assert open(fname).read() == "xxx"
         os.unlink(fname)
         self.interpret(f, [])
-        with open(fname) as fid:
-            assert fid.read() == "xxx"
+        assert open(fname).read() == "xxx"
 
     def test_fileno(self):
         fname = str(self.tmpdir.join('file_5'))
@@ -417,10 +411,7 @@ class TestFile(BaseRtypingTest):
         def f():
             with open(fname, "w") as f:
                 f.write("dupa")
-                assert not f.closed
-
             try:
-                assert f.closed
                 f.write("dupb")
             except ValueError:
                 pass
@@ -428,12 +419,10 @@ class TestFile(BaseRtypingTest):
                 assert False
 
         f()
-        with open(fname) as fid:
-            assert fid.read() == "dupa"
+        assert open(fname, "r").read() == "dupa"
         os.unlink(fname)
         self.interpret(f, [])
-        with open(fname) as fid:
-            assert fid.read() == "dupa"
+        assert open(fname, "r").read() == "dupa"
 
 
 class TestDirect:
@@ -478,8 +467,7 @@ class TestDirect:
                 s = ''.join([chr(32+(k&63)) for k in range(j, j + i)])
                 j += 1
                 print >> f, s
-        with open(fname) as fid:
-            expected = fid.readlines()
+        expected = open(fname).readlines()
         expected += ['', '']
         assert len(expected) == 252
 
@@ -511,16 +499,15 @@ class TestPopen(object):
             py.test.skip("not for win32")
 
     def test_popen(self):
-        f = rfile.create_popen_file("python -c 'print(42)'", "r")
+        f = rfile.create_popen_file("python -c 'print 42'", "r")
         s = f.read()
         f.close()
         assert s == '42\n'
 
-    @py.test.mark.skipif('sys.platform == "darwin"')
     def test_pclose(self):
         retval = 32
         printval = 42
-        cmd = "python -c 'import sys; print(%s); sys.exit(%s)'" % (
+        cmd = "python -c 'import sys; print %s; sys.exit(%s)'" % (
             printval, retval)
         f = rfile.create_popen_file(cmd, "r")
         s = f.read()
@@ -533,12 +520,10 @@ class TestPopenR(BaseRtypingTest):
     def setup_class(cls):
         if sys.platform == 'win32':
             py.test.skip("not for win32")
-        if sys.platform == 'darwin':
-            py.test.skip("not for os x")
 
     def test_popen(self):
         printval = 42
-        cmd = "python -c 'print(%s)'" % printval
+        cmd = "python -c 'print %s'" % printval
         def f():
             f = rfile.create_popen_file(cmd, "r")
             s = f.read()
@@ -549,7 +534,7 @@ class TestPopenR(BaseRtypingTest):
     def test_pclose(self):
         printval = 42
         retval = 32
-        cmd = "python -c 'import sys; print(%s); sys.exit(%s)'" % (
+        cmd = "python -c 'import sys; print %s; sys.exit(%s)'" % (
             printval, retval)
         def f():
             f = rfile.create_popen_file(cmd, "r")
