@@ -465,6 +465,15 @@ def test_connect_with_timeout_succeed():
     s.connect(INETAddress('python.org', 80))
     s.close()
 
+def test_connect_with_default_timeout_fail():
+    rsocket.setdefaulttimeout(0.1)
+    s = RSocket()
+    rsocket.setdefaulttimeout(None)
+    assert s.gettimeout() == 0.1
+    with py.test.raises(SocketTimeout):
+        s.connect(INETAddress('172.30.172.30', 12345))
+    s.close()
+
 def test_getsetsockopt():
     import struct
     assert struct.calcsize("i") == rffi.sizeof(rffi.INT)
