@@ -11,16 +11,10 @@ rootdir = os.path.dirname(os.path.dirname(os.path.abspath(sys.argv[0])))
 os.environ['PYTHONPATH'] = rootdir
 os.environ['PYTEST_PLUGINS'] = ''
 
-if sys.platform == 'win32':
-    pypyopt = "--pypy=pypy/goal/pypy3-c.exe"
-else:
-    pypyopt = "--pypy=pypy/goal/pypy3-c"
-
 popen = subprocess.Popen(
     [sys.executable, "pypy/test_all.py",
-     pypyopt,
-     "--timeout=1324",   # make it easy to search for
-     "-rs",
+     "--pypy=pypy/goal/pypy-c",
+     "--timeout=3600",
      "--duration=10",
      "--resultlog=cpython.log", "lib-python",
      ] + sys.argv[1:],
